/* AsciiObjectInputStream.java */


/*
A stream that allows us to read serialized objects as parsable ASCII.
This is taken from 
  Java Object Serialization in parsable ASCII format
  Enno Davids, Metva P/L.

  AUUG-Vic/CAUUG: Summer Chapter Technical Conference '98

  http://www.metva.com.au/users/enno/papers/java_ser.html
*/


package kb7.io;

import	java.io.*;
import	java.lang.reflect.*;
import	java.util.*;
 

/**
 * a stream that allows us to read serialized objects.
 */
public class AsciiObjectInputStream extends PushbackReader
{
  // the stream which is this one (mostly for handing to others).
  private Object		instance;	// handle to out stream object
  
  protected int		level;		// determine recursion depth.
  
  // the size of the biggest token we might push back.
  private final static int MAXTOKEN = 5;
  

  /**
   * create a new AsciiObjectInputStream
   */
  public AsciiObjectInputStream(
	BufferedReader	in
  ) 
  {
	// Hand the stream to PushbackReader to deal with.
	super( in, MAXTOKEN );
  
	// We have a bit of state ourselves which we want handled.
	this.instance = this;
	this.level = 0;
  }
  

  /**
   * create an object from the information in the stream.
   */
  public synchronized Object readAsciiObject()
  throws IOException
  {
	Object		obj;
	String		classToken = null;
	String		className = null;
	String		brace = null;
	boolean		scan;
  
	skipWhite();			// skip leading white space.
  
	// read till we see a "class" token...
	scan = true;
	do {
		classToken = readToken();
  
		if (isNull(classToken))
			return null;			// null class
		if (classToken.equals("class"))
			scan = false;
	} while (scan);
  
	className = readToken();
	if (isNull(className))
		return null;				// malformed.
  
	if (level > 0)
		readToken();				// skip varname.
	readToken();					// skip '='
  
	brace = readToken();
	if (isNull(brace))		// when we're called recursively
		return null;		// a field may be a null Object.
  
	try {
		Class		cl;
  
		// build an instance of our deserialized object.
		obj = Class.forName(className).newInstance();
  
		// Get a handle to the class of the object.
		cl = (obj instanceof Class) ?
				(Class)obj :
				obj.getClass();

/*  
		// We allow a class to provide its own encoding and
		//   decoding routines. Note that the AsciiObject
		//   interface isn't mandatory for our objects but it
		//   defines the writeAsciiObject() and
		//   readAsciiObject() methods for us to use here.
		if (obj instanceof AsciiObject) {
			return ((AsciiObject)obj).readAsciiObject(
				(AsciiObjectInputStream)instance );
			}
*/
  
		// recurse up the chain of superclasses and process
		//   those fields...
		read_superclass(obj, cl.getSuperclass());
  
		// ... and iterate along the fields declared here.
		read_fields(obj, obj);
  
		readToken();				// skip '}'
	}
	catch (Exception e) {
		throw new IOException(e.toString());
	}
  
	return obj;
  }
    

  void read_superclass(
	Object	obj,
	Class	cl
  ) throws Exception
  {
	if (cl != null) {
		read_superclass(obj, cl.getSuperclass());
		read_fields(obj, cl);
	}
  }
    

  void read_fields(
	Object	obj,
	Object	classobj
  ) throws Exception
  {
	Class		cl;
	Field		fields[];
  
	level++;
  
	// Get a handle to the class of the object.
	cl = (classobj instanceof Class) ?
			(Class)classobj :
			classobj.getClass();
  
	// detect when we've reached out limits. This is particularly
	//   the case when we're traversing the superclass chain.
	if ((cl == null) || (cl.isInstance(new Object()))) {
		level--;
		return;
	}
  
	// comment is silently skipped so nothing to do here.
  
	// process each field in turn.
	fields = cl.getDeclaredFields();
	for (int i=0 ; i<fields.length ; i++) {
		Class	ctype = fields[i].getType();
		int	mod;
		String	typeName = null,
			varName = null;
  
		mod = fields[i].getModifiers();
		if (Modifier.isStatic(mod))
			continue;
  
		// primitive types are handled directly.
		if (ctype.isPrimitive()) {
			typeName = readToken();
			varName = readToken();
			readToken();			// skip "="
  
			if (
				ctype.equals(Boolean.TYPE) ||
				ctype.equals(Character.TYPE) ||
				ctype.equals(Byte.TYPE) ||
				ctype.equals(Short.TYPE) ||
				ctype.equals(Integer.TYPE) ||
				ctype.equals(Long.TYPE) ||
				ctype.equals(Float.TYPE) ||
				ctype.equals(Double.TYPE)
			)
				handle_instance_field(	typeName,
							varName,
							obj,
							fields[i],
							readToken() );
			else if (ctype.equals(Void.TYPE))
				throw new IOException("read_fields: VOID type field found!");
			else
				throw new IOException("read_fields: unknown primitive type");
  
		// for arrays we need to extract the underlying type 1st
		} else if (ctype.isArray()) {
			StringTokenizer	st = null;
			int		dim;
			Object		nval = null;
  
			typeName = readToken();
			if (typeName.equals("class"))
				typeName = readToken();
			varName = readToken();
			readToken();		// skip "="
  
			st = new StringTokenizer(varName, "[]");
			st.nextToken();		// skip var name
  
                        String next = st.nextToken();

                        if ( next.equals("null") )
                                dim = -1;
                                // The array will be null. 
                        else
			        dim = Integer.parseInt(next);

			// no support for multi-dim arrays yet
			if (st.hasMoreTokens())
				throw new IOException("multi-dimensional array found - only one dimensional arrays supported");

                        // If the array is null, set the field accordingly.  
                        if ( dim == -1 ) {
                                fields[i].set(obj,null);
                                readToken();
                                // Skip the "null" used as its value.
                        }
			else {
         			readToken();		// skip curly brace

                                // if the constructor didn't make one for us...
			        if ( fields[i].get(obj) == null) {
					fields[i].set(obj, Array.newInstance(
						      ctype.getComponentType(),
					              dim));
			        }
  
				// pull in each element of the array
				for (int j=0 ; j<dim ; j++) {
					handle_array_field(	typeName,
								varName,
								obj,
								fields[i],
								j);
				}
				readToken();		// skip curly brace
			}
		} else {
  
			// Strings need special care
			if (ctype.isInstance(new String())) {
				typeName = readToken();
				varName = readToken();
				readToken();
  
				handle_instance_field(	typeName,
							varName,
							obj,
							fields[i],
							readString() );
  
			// recurse as everything else is another class
			} else {
				Object	nval = readAsciiObject();
  
				handle_instance_field(	typeName,
							varName,
							obj,
							fields[i],
							nval );
			}
		}
	}
  
	level--;
  }


  // pull in a single field which isn't an array or a non-string class
  void handle_instance_field(
	String	tname,
	String	fname,
	Object	obj,
	Field	fl,
	Object	value
  ) throws IOException
  {
	String		svalue = null;

	// Some sanity and 'do nothing' tests
	if (value == null)		return;

	// Convenience to save lots of casts later.
	if (value instanceof String) {
		svalue = (String)value;

		if (svalue.equals("null"))
			return;
	}

	// now try to run the assignments
	try {
		if (fl.getType().equals(Boolean.TYPE))
			fl.set(obj, new Boolean(svalue));
		else if (fl.getType().equals(Character.TYPE)) {
			char[]	onechar = new char[1];
			svalue.getChars(0, 1, onechar, 0);
			fl.set(obj, new Character(onechar[0]));
		} else if (fl.getType().equals(Byte.TYPE))
			fl.set(obj, new Byte(svalue));
		else if (fl.getType().equals(Short.TYPE))
			fl.set(obj, new Short(svalue));
		else if (fl.getType().equals(Integer.TYPE))
			fl.set(obj, new Integer(svalue));
		else if (fl.getType().equals(Long.TYPE))
			fl.set(obj, new Long(svalue));
		else if (fl.getType().equals(Float.TYPE))
			fl.set(obj, new Float(svalue));
		else if (fl.getType().equals(Double.TYPE))
			fl.set(obj, new Double(svalue));
		else if (fl.getType().equals((new String("").getClass())))
			fl.set(obj, value);
		else
			fl.set(obj, value);
	}
	catch (Exception e) {
		System.err.println(
			"ERROR: assigning to " +fl +"\n" +
			"\tread: " +tname +" " +fname +" = " +value);
		throw new IOException("field assignment failure:" + e);
	}
  }


  // pull in a single array element. This is similar to the above but
  //   we need to tell the reflect API which array element we're
  //   dealing with.
  void handle_array_field(
	String	tname,
	String	vname,
	Object	obj,
	Field	fl,
	int	idx
  ) throws IOException
  {
	Class	ctype = fl.getType();
	String	type = null,
		var = null,
		value = null;
  
	while (ctype.isArray())			// should only be 1 dim
		ctype = ctype.getComponentType();
  
	if (
		ctype.equals(Boolean.TYPE)	||
		ctype.equals(Character.TYPE)	||
		ctype.equals(Byte.TYPE)		||
		ctype.equals(Short.TYPE)	||
		ctype.equals(Integer.TYPE)	||
		ctype.equals(Long.TYPE)		||
		ctype.equals(Float.TYPE)	||
		ctype.equals(Double.TYPE)
	) {
		type = readToken();
		var = readToken();
		readToken();			// skip '='
		value = readToken();
	} else if ( ctype.equals((new String("").getClass())) ) {
		type = readToken();
		var = readToken();
		readToken();			// skip '='
		value = readString();
	}
  
	try {
		if ( ctype.equals(Boolean.TYPE) )
			Array.set(fl.get(obj), idx, new Boolean(value));
		else if ( ctype.equals(Character.TYPE) ) {
			char[]		onechar = new char[1];
			value.getChars(0, 1, onechar, 0);
			Array.set(fl.get(obj), idx, new Character(onechar[0]));
		} else if ( ctype.equals(Byte.TYPE) )
			Array.set(fl.get(obj), idx, new Byte(value));
		else if ( ctype.equals(Short.TYPE) )
			Array.set(fl.get(obj), idx, new Short(value));
		else if ( ctype.equals(Integer.TYPE) )
			Array.set(fl.get(obj), idx, new Integer(value));
		else if ( ctype.equals(Long.TYPE) )
			Array.set(fl.get(obj), idx, new Long(value));
		else if ( ctype.equals(Float.TYPE) )
			Array.set(fl.get(obj), idx, new Float(value));
		else if ( ctype.equals(Double.TYPE) )
			Array.set(fl.get(obj), idx, new Double(value));
		else if ( ctype.equals((new String("").getClass())) )
			Array.set(fl.get(obj), idx, value);
		else {
			Object	nobj = readAsciiObject();
  
			value = nobj == null ?
					null :
					nobj.toString();   // for diag.
			Array.set(fl.get(obj), idx, nobj);
		}
	}
	catch (Exception e) {
		System.err.println(
			"ERROR: assigning to " +fl +"[" +idx +"]\n" +
			"\tread: " +tname +" " +vname +" = " +value);
		e.printStackTrace(System.err);
  
		throw new IOException("field assignment failure:" + e);
	}
  }
  
  
  //------------------------------------------------------------------
  // UTILITY routines....
  //
  // From here on we have some utility routines which just package up
  //   the stuff we made use of earlier. That is input methods, and
  //   tests for what we're looking at.


  /**
   * create a String from the information in the stream.
   */
  String readString()
  throws IOException
  {
	String		token = null;
	StringBuffer	sb = new StringBuffer();
	boolean		quoted = false;
	int		ch;
  
	ch = read();
	if (ch == '"') {
		// sb.append((char)ch);		// leading '"'?
  
		do {
			ch = read();
  
			if (quoted) {
				switch (ch) {
				    case '"':
				    case '\\':
					sb.append((char)ch);
					break;
				    default:
					sb.append('\\');
					sb.append((char)ch);
					break;
				}
				quoted = false;
			} else {
				if (ch == '\\')
					quoted = true;
				else if (ch != '"')
					sb.append((char)ch);
			}
		} while ((ch != '"') || quoted);
		skipWhite();
  
		return sb.toString();
  
	} else if (ch == 'n') {
		unread(ch);
		token = readToken();
		if (token.equals("null")) {
			skipWhite();
			return null;
		}
	}
  
	throw new IOException("badly formatted string");
  }


  /**
   * return the next whitespace delimited token from the information
   *   in the stream.
   */
  String readToken()
  throws IOException
  {
	int		ch;
	StringBuffer	sb = new StringBuffer();
  
	ch = read();
	if (!isSpecial(ch)) {
		sb.append((char)ch);
  
		do {
			ch = read();
			if (!isWhite(ch))
				sb.append((char)ch);
		} while (!isWhite(ch));
	}
  
	// skip trailing whitespace.
	skipWhite();
  
	return sb.toString();
  }


  /**
   * Skip over one or more delimiter characters.
   */
  void skipWhite()
  throws IOException
  {
	int	ch;
	int	lastch = 0;
  
	do {
		ch = read();
		if ((ch=='/') && (lastch=='/')) {
			skipLine();
			ch = ' ';
		}
		lastch = ch;
	} while (isWhite(ch));
  
	unread(ch);
  }


  /*
   * Skip the rest of the line. Mainly used when we're processing C++
   *   double slash comments in the input stream.
   */
  void skipLine()
  throws IOException
  {
	int	ch;

	while ((ch = read()) != '\n');
  }


  /**
   * A couple of quick tests for what we got from the stream. Hardwired
   *   as there's no good reason to allow them to be configurable.
   */
  boolean isSpecial(
	int	ch
  ) {
	return (ch == '"');
  }
  
 
  boolean isWhite(
	int	ch
  ) {
	boolean	rv =  (
			(ch == '/') ||
			(ch == ' ') ||
			(ch == '\t') ||
			(ch == '\r') ||
			(ch == '\n')
		      );
  
	return rv;
  }
  

  boolean isNull(
	String	s
  ) {
	if (s == null)
		return true;
	if (s.equals("null"))
		return true;
   
	return false;
  }
}

  


