/* Dag.java */


package kb7.structures;


import java.util.Enumeration;
import java.util.Stack;
import java.util.Vector;
import kb7.structures.Map;


/**
 * This class implements a directed acyclic graph, and provides
 * a topological sort algorithm. We use it for checking data
 * dependencies in the languages we compile.
 */
public final class Dag
{
  private static final Object GREY = "Grey";
  private static final Object BLACK = "Black";
  // These represent states of the search tree during the
  // topological sort. We're just using them as unique tags
  // for two of the three possible states.


  private Map out_edges;
  // Maps a vertex to a list of the outgoing vertices, i.e.
  // represents the outgoing edges. out_edges.get(v) is
  // null if Vertex v is not in the graph, an empty Vector
  // if it is but has no outgoing edges, and a non-empty
  // Vector otherwise.


  public static void main( String argv[] )
  {
    Dag dag = new Dag();

    System.out.println( dag.toDisplay() );

    String a = "a", b = "b", c = "c", d = "d", e = "e", f = "f";

    dag.addVertex( a );
    dag.addEdge( a, b );
    dag.addEdge( b, f );
    dag.addEdge( c, d );
    dag.addVertex( e );
    System.out.println( dag.toDisplay() );

    /* Try a sort.
    */
    Vector sorted = null;
    try {
      sorted = dag.sort();
    }
    catch ( DagHasLoopException ex ) {
      System.out.println( ex.currentVertex );
    }
    System.out.println( sorted );

    /* Make a loop and try again.
    */
    dag.addEdge( f, a );
    System.out.println( dag.toDisplay() );
    try {
      sorted = dag.sort();
    }
    catch ( DagHasLoopException ex ) {
      System.out.println( ex.currentVertex );
    }
    System.out.println( sorted );

    /* Try sorting the empty graph.
    */
    dag = new Dag();
    System.out.println( dag.toDisplay() );
    try {
      sorted = dag.sort();
    }
    catch ( DagHasLoopException ex ) {
      System.out.println( ex.currentVertex );
    }
    System.out.println( sorted );
  }


  /**
   * Constructs a new empty graph.
   */
  public Dag()
  {
    this.out_edges = new Map();
  }


  /**
   * Adds a vertex a.
   */
  public final void addVertex( Object a )
  {
    if ( this.out_edges.get(a) == null )
      this.out_edges.put( a, new Vector() );
  }


  /**
   * Adds an edge from a to b.
   */
  public final void addEdge( Object a, Object b )
  {
    addVertex(a);
    addVertex(b);
    // That call ensures that this.out_edges.get(a) is
    // a Vector and not null, in case a hasn't already
    // been added. Similarly for b.
    ( (Vector)this.out_edges.get(a) ).addElement(b);
  }


  /**
   * Topologically sorts the graph, returning a vector of vertices.
   */
  public final Vector sort()
                      throws DagHasLoopException
  {
    /* This algorithm is standard, and is essentially a depth-first
       search which pushes the current vertex onto a stack at the start,
       and pops it at the end, adding it to the result vector. The
       search may not visit the entire graph when we start it from one
       vertex, and so we iterate round all vertices, searching from
       all those not already visited in previous searches.

       The idea for doing a topological sort this way is from (I think)
         Data Structures and Algorithms
         A V Aho, J E Hopcraft, J D Ullman
         Addison-Wesley 1982

       For detecting loops, we use the fact that vertices can
       be classified into white (not visited), grey (being visited), and
       black (finished with) during searching. Back arcs, i.e. those
       causing a loop, are indicated by encountering a grey vertex. This
       is explained in
         Introduction to Algorithms
         T H Cormen, C E Leiserson, R L Rivest
         MIT Press 1990
    */

    Vector results = new Vector();
    // To be set to the sequence of sorted vertices.

    Map visiteds = new Map();
    // Maps each vertex to either unvisited ("white", here null),
    // in process (GREY), or finished with (BLACK).

    Stack dones = new Stack();
    // We push the vertices onto this at the start of each call to
    // depth-first-search, transferring them to results at the end.

    Enumeration vertices = this.out_edges.domainElements();
    // All the vertices in our graph.

    while ( vertices.hasMoreElements() ) {
      Object v = vertices.nextElement();
      if ( visiteds.get(v) != BLACK )
        sortFromVertex( v, visiteds, dones, results );
        // v is either unvisited or grey. In the former case
        // "sort" will search it, in the latter report it.
    }

    return results;
  }

  /* Does a depth-first search from vertex v.
  */
  private final void sortFromVertex( Object v, Map visiteds,
                                     Stack dones, Vector results )
                     throws DagHasLoopException
  {
    if ( visiteds.get(v)==GREY )
      throw new DagHasLoopException( v );

    dones.push( v );

    visiteds.put( v, GREY );

    Vector out_edges = (Vector)this.out_edges.get(v);
    for ( int i=0; i<out_edges.size(); i++ ) {
      Object w = out_edges.elementAt(i);
      if ( visiteds.get(w) != BLACK )
        sortFromVertex( w, visiteds, dones, results );
    }

    visiteds.put( v, BLACK );

    results.addElement( dones.pop() );
  }


  public final String toDisplay()
  {
    String s = "{ " ;
    Enumeration vertices = this.out_edges.domainElements();
    while ( vertices.hasMoreElements() ) {
      Object v = vertices.nextElement();
      s += ( v + " --> (" );
      Vector out_edges = (Vector)this.out_edges.get(v);
      for ( int i=0; i<out_edges.size(); i++ ) {
        s += out_edges.elementAt(i);
        if ( i < out_edges.size()-1 )
          s += ",";
      }
      s += ") ";
    }
  return s + "}";
  }


  public final String toString()
  { return "[Dag " + toDisplay() + "]"; }

}
