/* Dimension.java */


package kb7.structures;

import kb7.general.Bug;


/**
 * This class implements dimensional analysis.
 * <P>
 * Each instance represents a "dimension", such as mass or length or
 * speed. Instances contain an array of exponents and an array of names,
 * and the ith element of the exponents array is the exponent
 * corresponding to the ith element of the names array.
 * <P>
 * There are methods for multiplying and dividing dimensions, and
 * checking whether two Dimensions have the same exponents.
 * <P>
 * There are three constructors. One takes an array of names and creates
 * a null Dimension, with all exponents zero. The second takes an array
 * of names and one of exponents, and creates a Dimension with both.
 * The third takes an array of names and a name, searches the one for
 * the other, and creates the corresponding Dimension.
 * <P>
 * When using Dimensions together, they must be "compatible". This means
 * that they must have the same length exponent arrays, and that the
 * name arrays must be pointer identical (not just equal in value).
 * We assume that when you create a set of dimensions, you will give the
 * same name array to all of them. The methods that create new
 * Dimensions just copy these along.
 */
public class Dimension implements Cloneable
{
  private int[] exponents;
  // The ith element is the exponent of the ith dimension.

  private String[] names;
  // The ith element is the name of the ith dimension.


  public static void main( String argv[] )
  {
    String[] names = { "M", "L", "T" };

    int[] m = { 1, 0, 0 };
    Dimension mass = new Dimension( names, m );

    int[] l = { 0, 1, 0 };
    Dimension length = new Dimension( names, "L" );

    int[] t = { 0, 0, 1 };
    Dimension time = new Dimension( names, t );

    Dimension speed = length.quotient( time );
    Dimension acceleration = speed.quotient( time );
    Dimension force = mass.product( acceleration );
    Dimension energy = force.product( length );
    Dimension area = length.exp(2);
    Dimension none = energy.quotient( energy );
    Dimension none2 = new Dimension();
    Dimension none3 = none2.product( new Dimension() );
    Dimension length2 = none2.product( length );
    Dimension length3 = length.product( none2 );

    System.out.println( "Length = " + length.toDisplay() );
    System.out.println( "Time = " + time.toDisplay() );
    System.out.println( "Speed = " + speed.toDisplay() );
    System.out.println( "Acceleration = " + acceleration.toDisplay() );
    System.out.println( "Force = " + force.toDisplay() );
    System.out.println( "Energy = " + energy.toDisplay() );
    System.out.println( "Area = " + area.toDisplay() );
    System.out.println( "None = " + none.toDisplay() );
    System.out.println( "None2 = " + none2.toDisplay() );
    System.out.println( "None3 = " + none3.toDisplay() );
    System.out.println( "Length2 = " + length2.toDisplay() );
    System.out.println( "Length3 = " + length3.toDisplay() );

    if ( length.equals(time) )
      System.out.println( "Length = time" );
    else
      System.out.println( "Length != time" );

    if ( length.equals(speed.product(time)) )
      System.out.println( "Length = speed*time" );
    else
      System.out.println( "Length != speed*time" );

    if ( energy.quotient(energy).equals( force.quotient(mass.product(acceleration)) ) )
      System.out.println( "Energy/energy = force/(mass*acceleration)" );
    else
      System.out.println( "Energy/energy != force/(mass*acceleration)" );
  }


  /**
   * Creates a new Dimension with the specified names and exponents.
   * Throws an exception if the number of names and exponents are not
   * the same.
   */
  public Dimension( String[] names, int[] exponents )
  {
    /* Check compatibility of arguments. */
    if ( names.length != exponents.length )
      throw new Bug( "Name and exponent arrays not the same length:" + names + " " + exponents );
    this.names = names;
    this.exponents = exponents;
  }


  /**
   * Creates a new Dimension all of whose exponents are zero.
   */
  public Dimension( String[] names )
  {
    this.names = names;
    this.exponents = new int[names.length];
    for ( int i=0; i<names.length; i++ )
      this.exponents[i] = 0;
  }


  /**
   * Creates a new Dimension with the i'th exponent 1 and all
   * others zero. i is based at 0, for compatibility with "names".
   */
  public Dimension( String[] names, int i )
  {
    this( names );
    this.exponents[i] = 1;
  }


  /**
   * Creates a new Dimension corresponding to the position of
   * "name" in "names".
   */
  public Dimension( String[] names, String name )
  {
    boolean found = false;
    int[] exponents = new int[names.length];
    for ( int i=0; i<names.length; i++ ) {
      if ( names[i].equals(name) ) {
        exponents[i] = 1;
        found = true;
      }
      else
        exponents[i] = 0;
    }
    if ( ! found )
      throw new Bug( "Name not found in name array: " + names + " " + name );
    this.names = names;
    this.exponents = exponents;
  }


  /**
   * Creates a new Dimension representing a scalar. This can also be done
   * by setting all exponents to zero. However, this constructor is the
   * one case where the names array need not be set, so it enables zero
   * dimensions to be created before we know what the names are. The arithmetic
   * procedures all treat dimensions with null name arrays specially.
   */
  public Dimension()
  {
    this.names = null;
    this.exponents = new int[0];
  }


  /**
   * Multiplies this by d2.
   */
  public final void multiply( Dimension d2 )
  {
    checkCompatible( this, d2 );
    for ( int i=0; i<this.exponents.length; i++ )
      this.exponents[i] += d2.exponents[i];
  }


  /**
   * Divides this by d2.
   */
  public final void divide_by( Dimension d2 )
  {
    checkCompatible( this, d2 );
    for ( int i=0; i<this.exponents.length; i++ )
      this.exponents[i] -= d2.exponents[i];
  }


  /**
   * Multiples all exponents in this by power, in effect
   * raising to that power.
   */
  public final void raise_to( int power )
  {
    for ( int i=0; i<this.exponents.length; i++ )
      this.exponents[i] = this.exponents[i] * power;
  }


  /**
   * Returns the product of this with d2.
   */
  public final Dimension product( Dimension d2 )
  {
    Dimension result = (Dimension)this.clone();
    result.multiply( d2 );
    return result;
  }


  /**
   * Returns the quotient, this divided by d2.
   */
  public final Dimension quotient( Dimension d2 )
  {
    Dimension result = (Dimension)this.clone();
    result.divide_by( d2 );
    return result;
  }


  /**
   * Returns the result of raising to power.
   */
  public final Dimension exp( int power )
  {
    Dimension result = (Dimension)this.clone();
    result.raise_to( power );
    return result;
  }


  /* Checks that the names are the same for both arguments, and
     the exponent arrays are the same length. Used internally.
  */
  private static final void checkCompatible( Dimension d1, Dimension d2 )
  {
    if ( d1.names == null && d2.names == null )
      return;
    else if ( d1.names == null && d2.names != null ) {
      d1.names = d2.names;
      d1.exponents = new int[d2.names.length];
      for ( int i=0; i<d2.names.length; i++ )
        d1.exponents[i] = 0;
      return;
    }
    else if ( d2.names == null ) {
      d2.names = d1.names;
      d2.exponents = new int[d1.names.length];
      for ( int i=0; i<d1.names.length; i++ )
        d2.exponents[i] = 0;
      return;
    }

    if ( d1.names != d2.names )
      throw new Bug( "Names not the same in Dimensions: " + d1 + " " + d1 );
    if ( d1.exponents.length != d2.exponents.length )
      throw new Bug( "Exponent array sizes not the same in Dimensions: " + d1 + " " + d2 );
  }


  /**
   * True if its argument has the same exponents as this. Overrides
   * the Object.equals method by allowing the exponent arrays to
   * be equal in content rather than insisting that d2 be pointer-identical
   * to this.
   */
  public final boolean equals( Object d2 )
  {
    if ( ! (d2 instanceof Dimension) )
      throw new Bug( "Argument is not a Dimension: " + d2 );
    checkCompatible( this, (Dimension)d2 );
    for ( int i=0; i<this.exponents.length; i++ )
      if ( this.exponents[i] != ((Dimension)d2).exponents[i] )
        return false;
    return true;
  }


  /**
   * True if this is of zero dimension.
   */
  public final boolean isZero()
  { return equals( new Dimension() ); }


  /**
   * Overrides the Object.clone method, by copying exponent
   * array. Does not copy the names, since we assume these must be
   * identical between all Dimensions over the same basis.
   */
  public final Object clone()
  {
    if ( this.names == null )
      return new Dimension();
    else
      return new Dimension( this.names, (int[])this.exponents.clone() );
  }


  /**
   * Returns a Dimension's string representation in a concise easy-to-
   * read format.
   */
  public final String toDisplay()
  {
    String s = "";
    // The result.

    if ( this.names == null )
      return "0";

    int zero_exponents = 0;
    // Counts the number of exponents that were zero. We don't
    // output anything for these, and if they were all zero, s
    // would be empty, so we need to output "none" or something
    // similar.

    for ( int i=0; i<this.exponents.length; i++ ) {
      if ( this.exponents[i] == 0 )
        zero_exponents++;
      else {
        /* We want " * " between items in s, but not before the first
           or after the last. This should ensure that.
        */
        if ( !s.equals("") )
          s += " * ";
        if ( this.exponents[i] == 1 )
          s += this.names[i];
        else if ( this.exponents[i] > 1 || this.exponents[i] < 0 )
          s += this.names[i] + "^" + (this.exponents[i] + "");
        else
          throw new Bug( "Illegal condition" );
      }
    }
    if ( zero_exponents == this.exponents.length )
      s = "0";
    return s;
  }


  /**
   * Returns a Dimension's string representation, including the
   * class tag. Mainly for diagnostics.
   */
  public final String toString()
  {
    return "[" + getClass().toString() + " " + toDisplay() + "]";
  }
}
