/*  IdGenerator.java  */


package kb7.general;
import java.util.Date;


/**
 * The class IdGenerator is used for generating strings that can be
 * used as unique filenames. They are generated to base 36 (letters
 * and digits) - this means that if the strings are used as filenames,
 * they're safe, whilst still being reasonably compact.
 */
public final class IdGenerator
{
  /* We take all dates relative to the start of 1999. This makes
     the times, and hence the ids, shorter.
     Note that years are specified as the number of years since 1900.
  */
  private final static long start_from_time = new Date( 99, 1, 1 ).getTime();


  /** Generates and returns a new unique id.
  */
  public final static String generate()
  {
    /* Get the current time (milliseconds since 1/1/1970. */
    long time = new Date().getTime();

    /* Ensure uniqueness by waiting until the next id
       generated from the time will be different.
    */

    while ( new Date().getTime() == time )
      ;

    /* Subtract a nearby starting point from the time to make
       the strings smaller.
    */
    return numberToId( time-start_from_time );

  }


  /* Converts an integer to an id. We do this by rewriting in base
     36.
  */

  private final static String numberToId( long n )
  {
    /* Very simple in Java - using 36 as the radix argument to toString
       appears to depict the number using lower-case letters and digits.
    */
    return Long.toString( n, 36 );
  }

}
