/* Tables.java */


package kb7.html;

import java.text.NumberFormat;
import java.util.Vector;
import kb7.general.Bug;
import kb7.html.Table;


/**
 * This class provides some utilities for copying
 * data, including arrays and vectors, into rows and
 * columns of the kb7.html.Table class.
 *
 * Methods:
 *  copyCell( t, x, y, datum )
 *    Copy datum's string representation into the cell at (x,y).
 *    Cell coordinates are based at 1, as for the Table class.
 *
 *  copyRow( t, x, y, array or vector )
 *    Copy elements from array or vector to the cells at (x,y),
 *    (x+1,y) etc.
 *
 *  copyColumn( t, x, y, array or vector )
 *    Copy elements from array or vector to the cells at (x,y),
 *    (x,y+1) etc.
 *
 *  copyRows( t, x, y, array )
 *    array is a 2-d array whose first index iterates over 
 *    successive rows. Copy the first row, array[0], to
 *    (x,y), the second to (x,y+1), etc.
 *
 *  copyColumns( t, x, y, array )
 *    array is a 2-d array whose first index iterates over 
 *    successive columns. Copy the first column, array[0], to
 *    (x,y), the second to (x+1,y), etc.
 *
 *  insertRowAfter( t, y )
 *    Insert a new row after row y.
 *
 *  insertColumnAfter( t, x )
 *    Insert a new column after column x.
 *
 * There are variants of the copy... methods taking double arguments
 * which also take a NumberFormat, and use that to convert the number
 * to a string when copying it into a cell. There are also variants
 * which take an attributes argument before the string, and copy
 * into the cell's attributes part.
 */
public final class Tables
{
  public final static void copyCell( Table t, int x, int y, int i )
  {
    t.setCell( i+"", x, y ); 
  }

  public final static void copyCell( Table t, int x, int y, String attr, int i )
  {
    t.setCell( attr, i+"", x, y ); 
  }

  public final static void copyCell( Table t, int x, int y, double d )
  {
    t.setCell( d+"", x, y ); 
  }

  public final static void copyCell( Table t, int x, int y, String attr, double d )
  {
    t.setCell( attr, d+"", x, y ); 
  }

  public final static void copyCell( Table t, int x, int y, double d, NumberFormat f )
  {
    t.setCell( f.format(d), x, y ); 
  }

  public final static void copyCell( Table t, int x, int y, String attr, double d, NumberFormat f )
  {
    t.setCell( attr, f.format(d), x, y ); 
  }

  public final static void copyCell( Table t, int x, int y, Object o )
  {
    t.setCell( o.toString(), x, y ); 
  }

  public final static void copyCell( Table t, int x, int y, String attr, Object o )
  {
    t.setCell( attr, o.toString(), x, y ); 
  }

  public final static void copyCell( Table t, int x, int y, Table src, int x_, int y_ )
  {    
    Cell cell = src.getCell( x_, y_ );
    t.setCell( cell.getAttributes(), cell.getContents(), x, y );
  }
 

  public final static void copyRow( Table t, int x, int y, Vector v )
  {
    for ( int i=0; i<v.size(); i++ )
      copyCell( t, x+i, y, v.elementAt(i) );
  }

  public final static void copyRow( Table t, int x, int y, String attr, Vector v )
  {
    for ( int i=0; i<v.size(); i++ )
      copyCell( t, x+i, y, attr, v.elementAt(i) );
  }

  public final static void copyRow( Table t, int x, int y, int[] a )
  {
    for ( int i=0; i<a.length; i++ )
      copyCell( t, x+i, y, a[i] );
  }  

  public final static void copyRow( Table t, int x, int y, String attr, int[] a )
  {
    for ( int i=0; i<a.length; i++ )
      copyCell( t, x+i, y, attr, a[i] );
  }

  public final static void copyRow( Table t, int x, int y, int[] a, NumberFormat f )
  {
    for ( int i=0; i<a.length; i++ )
      copyCell( t, x+i, y, a[i], f );
  }

  public final static void copyRow( Table t, int x, int y, String attr, int[] a, NumberFormat f )
  {
    for ( int i=0; i<a.length; i++ )
      copyCell( t, x+i, y, attr, a[i], f );
  }

  public final static void copyRow( Table t, int x, int y, double[] a )
  {
    for ( int i=0; i<a.length; i++ )
      copyCell( t, x+i, y, a[i] );
  }

  public final static void copyRow( Table t, int x, int y, String attr, double[] a )
  {
    for ( int i=0; i<a.length; i++ )
      copyCell( t, x+i, y, attr, a[i] );
  }

  public final static void copyRow( Table t, int x, int y, double[] a, NumberFormat f )
  {
    for ( int i=0; i<a.length; i++ )
      copyCell( t, x+i, y, a[i], f );
  }

  public final static void copyRow( Table t, int x, int y, String attr, double[] a, NumberFormat f )
  {
    for ( int i=0; i<a.length; i++ )
      copyCell( t, x+i, y, attr, a[i], f );
  }

  public final static void copyRow( Table t, int x, int y, Object[] a )
  {
    for ( int i=0; i<a.length; i++ )
      copyCell( t, x+i, y, a[i] );
  }

  public final static void copyRow( Table t, int x, int y, String attr, Object[] a )
  {
    for ( int i=0; i<a.length; i++ )
      copyCell( t, x+i, y, attr, a[i] );
  }


  public final static void copyColumn( Table t, int x, int y, Vector v )
  {
    for ( int i=0; i<v.size(); i++ )
      copyCell( t, x, y+i, v.elementAt(i) );
  }

  public final static void copyColumn( Table t, int x, int y, String attr, Vector v )
  {
    for ( int i=0; i<v.size(); i++ )
      copyCell( t, x, y+i, attr, v.elementAt(i) );
  }

  public final static void copyColumn( Table t, int x, int y, int[] a )
  {
    for ( int i=0; i<a.length; i++ )
      copyCell( t, x, y+i, a[i] );
  }  

  public final static void copyColumn( Table t, int x, int y, String attr, int[] a )
  {
    for ( int i=0; i<a.length; i++ )
      copyCell( t, x, y+i, attr, a[i] );
  }

  public final static void copyColumn( Table t, int x, int y, int[] a, NumberFormat f )
  {
    for ( int i=0; i<a.length; i++ )
      copyCell( t, x, y+i, a[i], f );
  }

  public final static void copyColumn( Table t, int x, int y, String attr, int[] a, NumberFormat f )
  {
    for ( int i=0; i<a.length; i++ )
      copyCell( t, x, y+i, attr, a[i], f );
  }

  public final static void copyColumn( Table t, int x, int y, double[] a )
  {
    for ( int i=0; i<a.length; i++ )
      copyCell( t, x, y+i, a[i] );
  }

  public final static void copyColumn( Table t, int x, int y, String attr, double[] a )
  {
    for ( int i=0; i<a.length; i++ )
      copyCell( t, x, y+i, attr, a[i] );
  }

  public final static void copyColumn( Table t, int x, int y, double[] a, NumberFormat f )
  {
    for ( int i=0; i<a.length; i++ )
      copyCell( t, x, y+i, a[i], f );
  }

  public final static void copyColumn( Table t, int x, int y, String attr, double[] a, NumberFormat f )
  {
    for ( int i=0; i<a.length; i++ )
      copyCell( t, x, y+i, attr, a[i], f );
  }

  public final static void copyColumn( Table t, int x, int y, Object[] a )
  {
    for ( int i=0; i<a.length; i++ )
      copyCell( t, x, y+i, a[i] );
  }

  public final static void copyColumn( Table t, int x, int y, String attr, Object[] a )
  {
    for ( int i=0; i<a.length; i++ )
      copyCell( t, x, y+i, attr, a[i] );
  }

  
  public final static void copyRows( Table t, int x, int y, int[][] a )
  { 
    for ( int i=0; i<a.length; i++ )
      copyRow( t, x, y+i, a[i] );
  }

  public final static void copyRows( Table t, int x, int y, String attr, int[][] a )
  { 
    for ( int i=0; i<a.length; i++ )
      copyRow( t, x, y+i, attr, a[i] );
  }

  public final static void copyRows( Table t, int x, int y, double[][] a )
  { 
    for ( int i=0; i<a.length; i++ )
      copyRow( t, x, y+i, a[i] );
  }

  public final static void copyRows( Table t, int x, int y, String attr, double[][] a )
  { 
    for ( int i=0; i<a.length; i++ )
      copyRow( t, x, y+i, attr, a[i] );
  }

  public final static void copyRows( Table t, int x, int y, double[][] a, NumberFormat f )
  { 
    for ( int i=0; i<a.length; i++ )
      copyRow( t, x, y+i, a[i], f );
  }

  public final static void copyRows( Table t, int x, int y, String attr, double[][] a, NumberFormat f )
  { 
    for ( int i=0; i<a.length; i++ )
      copyRow( t, x, y+i, attr, a[i], f );
  }

  public final static void copyRows( Table t, int x, int y, Object[][] a )
  { 
    for ( int i=0; i<a.length; i++ )
      copyRow( t, x, y+i, a[i] );
  }

  public final static void copyRows( Table t, int x, int y, String attr, Object[][] a )
  { 
    for ( int i=0; i<a.length; i++ )
      copyRow( t, x, y+i, attr, a[i] );
  }

  public final static void copyColumns( Table t, int x, int y, int[][] a )
  { 
    for ( int i=0; i<a.length; i++ )
      copyColumn( t, x+i, y, a[i] );
  }  

  public final static void copyColumns( Table t, int x, int y, String attr, int[][] a )
  { 
    for ( int i=0; i<a.length; i++ )
      copyColumn( t, x+i, y, attr, a[i] );
  }  

  public final static void copyColumns( Table t, int x, int y, double[][] a )
  { 
    for ( int i=0; i<a.length; i++ )
      copyColumn( t, x+i, y, a[i] );
  } 

  public final static void copyColumns( Table t, int x, int y, String attr, double[][] a )
  { 
    for ( int i=0; i<a.length; i++ )
      copyColumn( t, x+i, y, attr, a[i] );
  }

  public final static void copyColumns( Table t, int x, int y, double[][] a, NumberFormat f )
  { 
    for ( int i=0; i<a.length; i++ )
      copyColumn( t, x+i, y, a[i], f );
  }

  public final static void copyColumns( Table t, int x, int y, String attr, double[][] a, NumberFormat f )
  { 
    for ( int i=0; i<a.length; i++ )
      copyColumn( t, x+i, y, attr, a[i], f );
  }

  public final static void copyColumns( Table t, int x, int y, Object[][] a )
  { 
    for ( int i=0; i<a.length; i++ )
      copyColumn( t, x+i, y, a[i] );
  }  

  public final static void copyColumns( Table t, int x, int y, String attr, Object[][] a )
  { 
    for ( int i=0; i<a.length; i++ )
      copyColumn( t, x+i, y, attr, a[i] );
  }  


  public final static void insertRowAfter( Table t, int y )
  {
    t.insertRowAfter( y );
  }

  public final static void insertColumnAfter( Table t, int x )
  {
    t.insertColumnAfter( x );
  }


  public final static void mergeSubtablesAlongColumn( Table old, int x, int y )
  {
    Table first = (Table)old.getCell( x, y ).getContents();
    // The first subtable.

    int nrows = first.getWidth();
    int ncols = first.getHeight();
    // Rows and columns in the subtables. We assume each has the same size.

    int xmax = old.getWidth();
    int ymax = old.getHeight();
    // Size of original table.

    Table neww = new Table( xmax+ncols-1, (ymax-y+1)*nrows + (y-1) );
    // Table to copy into.

    /* Do left of multicolumn and above. */
    for ( int k=1; k<x; k++ ) {
      for ( int l=1; l<=ymax; l++ ) {
        int K = k;
        int L = l;
        copyCell( neww,K,L, old,k,l );
      }
    }

    /* Do left of multicolumn and below. */
    for ( int k=1; k<x; k++ ) {
      for ( int l=1; l<=ymax; l++ ) {
        int K = k;
        int L = y + (l-y) * nrows;
        copyCell( neww,K,L, old,k,l );
      }
    }

    /* Do above multicolumn. */
    for ( int l=1; l<y; l++ ) {
      int K = x;
      int L = l;
      old.setCellColspan( K, L, ncols );
      copyCell( neww,K,L, old,x,l );
    }

    /* Do right of multicolumn and above. */
    for ( int k=x+1; k<=xmax; k++ ) {
      for ( int l=1; l<=ymax; l++ ) {
        int K = k + ncols-1;
        int L = l;
        copyCell( neww,K,L, old,k,l );
      }
    }

    /* Do right of multicolumn and below. */
    for ( int k=x+1; k<=xmax; k++ ) {
      for ( int l=1; l<=ymax; l++ ) {
        int K = k + ncols-1;
        int L = y + (l-y) * nrows;
        old.setCellRowspan( K, L, nrows );
        copyCell( neww,K,L, old,k,l );
      }
    }       
  
    /* Do multicolumn. */
    for ( int l=y; l<=ymax; l++ ) {
      int K0 = x;
      int L0 = y + (l-y) * nrows ;
      Table subtable = (Table)old.getCell( x, l ).getContents();
      for ( int i=1; i<=ncols; i++ ) 
        for ( int j=1; j<=nrows; j++ ) 
          copyCell( neww,K0+i-1,L0+j-1, subtable,i,j );
    }
  }
}
