/* Html.java */


package kb7.html;

import java.util.Vector;
import javax.servlet.*;
import javax.servlet.http.*;


/**
 * This class provides a collection of functions for generating HTML.
 * Using these makes it easier to construct things such as complicated
 * tables.
 * <P> 
 * All the functions are static. Most generate containers (HTML
 * constructs with opening and closing tags) and take two string
 * arguments. In these, the first argument can be used to insert
 * stuff inside the opening tag; the second to insert it between
 * the tags.
 * <P>
 * Functions with names ending NV generate input elements, where the
 * NAME= and VALUE= attributes are passed as separate arguments.
 * These are convenient when the caller needs to generate names and
 * values dynamically, as it avoids messing around with quotes.
 */
public class Html
{  
  private final static String eol = System.getProperty( "line.separator", "\n" );


/*
The methods below generate table components.
*/


  /**
   * Generates a TABLE. s goes between the opening and closing tags.
   */
  public static String table( String s )
  {
    return "<TABLE>" + eol + s + "</TABLE>" + eol;
  }

  /**
   * Generates a TABLE. s2 goes between the opening and closing tags;
   * s1 inserts arguments inside the opening tag.
   */
  public static String table( String s1, String s2 )
  {
    return "<TABLE " + s1 + ">" + eol + s2 + "</TABLE>" + eol;
  }


  /**
   * Generates a table row, TR. s goes between the opening and closing tags.
   */
  public static String tr( String s )
  {
    return "<TR>" + s + "</TR>" + eol;
  }

  /**
   * Generates a table row, TR. s2 goes between the opening and closing tags;
   * s1 inserts arguments inside the opening tag.
   */
  public static String tr( String s1, String s2 )
  {
    return "<TR " + s1 + ">" + s2 + "</TR>" + eol;
  }


  /**
   * Generates a table cell, TH. s goes between the opening and closing tags.
   */
  public static String th( String s )
  {
    return "<TH>" + s + "</TH>";
  }

  /**
   * Generates a table cell, TH. s2 goes between the opening and closing tags;
   * s1 inserts arguments inside the opening tag.
   */
  public static String th( String s1, String s2 )
  {
    return "<TH " + s1 + ">" + s2 + "</TH>";
  }


  /**
   * Generates a table cell, TD. s goes between the opening and closing tags.
   */
  public static String td( String s )
  {
    return "<TD>" + s + "</TD>";
  }  

  /**
   * Generates a table cell, TD. s2 goes between the opening and closing tags;
   * s1 inserts arguments inside the opening tag.
   */
  public static String td( String s1, String s2 )
  {
    return "<TD " + s1 + ">" + s2 + "</TD>";
  }


/*
The methods below generate input elements.
*/


  /**
   * Generates a text input field. s inserts arguments inside the tag. 
   * There is no two-argument version of this method.
   */
  public static String textField( String s )
  {
    return "<INPUT TYPE=TEXT " + s + ">";
  }


  /**
   * Generates a hidden field. s inserts arguments inside the tag. 
   * There is no two-argument version of this method.
   */
  public static String hiddenField( String s )
  {
    return "<INPUT TYPE=HIDDEN " + s + ">";
  }


  /**
   * Generates a SUBMIT button. s inserts arguments inside the tag. 
   * There is no two-argument version of this method.
   */
  public static String submitButton( String s )
  {
    return "<INPUT TYPE=SUBMIT " + s + ">";
  }


 /**
   * Generates a radio button. s inserts arguments inside the tag. 
   * There is no two-argument version of this method.
   */
  public static String radioButton( String s )
  {
    return "<INPUT TYPE=RADIO " + s + ">";
  }


 /**
   * Generates a check box. s inserts arguments inside the tag. 
   * There is no two-argument version of this method.
   */
  public static String checkBox( String s )
  {
    return "<INPUT TYPE=CHECKBOX " + s + ">";
  }


 /**
   * Generates a TEXTAREA. s2 goes between the opening and closing tags;
   * s1 inserts arguments inside the opening tag.
   */
  public static String textArea( String s1, String s2 )
  {
    return "<TEXTAREA " + s1 + ">" + s2 + "</TEXTAREA>";
  }


 /**
   * Generates a menu. s2 goes between the opening and closing tags;
   * s1 inserts arguments inside the opening tag.
   */
  public static String menu( String s1, String s2 )
  {
    return "<SELECT " + s1 + ">" + s2 + "</SELECT>";
  }


  /**
   * Generates a menu option containing s.
   */
  public static String option( String s )
  {
    return "<OPTION>" + s;
  }


  /**
   * Same as option(s).
   */
  public static String options( String s )
  {
    return option(s);
  }
 

  /**
   * Generates a sequence of menu options running from min to max
   * inclusive by step.
   */
  public static String options( int min, int max, int step )
  {
    String options = "";
    for ( int i=min; i<=max; i=i+step ) {
      options += ("<OPTION>" + i);
    }
    return options;
  }


  /**
   * Generates a sequence of menu options running from min to max
   * inclusive by step. If one of them is value, it will be selected.
   */
  public static String options( int min, int max, int step, int value )
  {
    String options = "";
    for ( int i=min; i<=max; i=i+step ) {
      if ( i==value)
        options += "<OPTION SELECTED>" + i;
      else
        options += "<OPTION>" + i;
    }
    return options;
  }


  /**
   * Generates a sequence of menu options running over the
   * elements of v.
   */
  public static String options( Vector v )
  {
    String options = "";
    for ( int i=0; i<v.size(); i++ ) {
      options += ( "<OPTION>" + v.elementAt(i) );
    }
    return options;
  }


  /**
   * Generates a sequence of menu options running over the
   * elements of v. If one of them is value, it will be
   * selected.
   */
  public static String options( Vector v, Object value )
  {
    String options = "";
    for ( int i=0; i<v.size(); i++ ) {
      Object option = v.elementAt(i);
      if ( option.equals(value))
        options += "<OPTION SELECTED>" + option;
      else
        options += "<OPTION>" + option;
    }
    return options;
  }


  /**
   * Generates a sequence of menu options running over the
   * elements of a.
   */
  public static String options( String[] a )
  {
    String options = "";
    for ( int i=0; i<a.length; i++ ) {
      options += ( "<OPTION>" + a[i] );
    }
    return options;
  }


  /**
   * Generates a sequence of menu options running over the
   * elements of v. If one of them is value, it will be
   * selected.
   */
  public static String options( String[] a, Object value )
  {
    String options = "";
    for ( int i=0; i<a.length; i++ ) {
      Object option = a[i];
      if ( option.equals(value))
        options += "<OPTION SELECTED>" + option;
      else
        options += "<OPTION>" + option;
    }
    return options;
  }


/*
The methods below generate headings.
*/


  /**
   * Generates an Hn heading with s1 inside the opening tag, and
   * s2 between the tags.
   */
  public static String h( int n, String s1, String s2 )
  {
    return ("<H" + n + " " + s1 + ">") + s2 + ("</H" + n + ">");
  }


/*
The methods below generate links.
*/


  /**
   * Generates an <A HREF> link.
   */
  public static String aHref( String href, String contents )
  {
    return "<A HREF=\"" + href + "\">" + contents + "</A>";
  }


  /**
   * Generates an <A NAME> link.
   */
  public static String aName( String name, String contents )
  {
    return "<A NAME=\"" + name + "\">" + contents + "</A>";
  }


/*
The methods below are the NV ones.
*/


  /**
   * Returns a text field.
   */
  public static String textFieldNV( String name, String value, String attributes )
  {
    return textField( "NAME=\"" + name + "\" VALUE=\"" + value + "\" " + attributes ); 
  }  


  /**
   * Returns a text area.
   */
  public static String textAreaNV( String name, String value, String attributes )
  {
    return textArea( "NAME=\"" + name + "\"" + attributes, value ); 
  }


  /**
   * Returns a menu.
   */
  public static String menuN( String name, String options, String attributes )
  {
    return menu( "NAME=\"" + name + "\"" + attributes, options ); 
  }


  /**
   * Returns a hidden field.
   */
  public static String hiddenFieldNV( String name, String value )
  {
    return hiddenField( "NAME=\"" + name + "\" VALUE=\"" + value + "\" " );
  }  


  /**
   * Returns a SUBMIT button.
   */
  public static String submitButtonNV( String name, String value )
  {
    return submitButton( "NAME=\"" + name + "\" VALUE=\"" + value + "\" " );
  }  


  /**
   * Returns a radio button.
   */
  public static String radioButtonNV( String name, String value, String attributes, boolean checked )
  { 
    if ( checked )
      attributes += " CHECKED";
    String s = radioButton( "NAME=\"" + name + "\" VALUE=\"" + value + "\" " + attributes );   
    return s;
  }  


  /**
   * Returns a checkbox.
   */
  public static String checkBoxNV( String name, String value, String attributes, boolean checked )
  { 
    if ( checked )
      attributes += " CHECKED";
    String s = checkBox( "NAME=\"" + name + "\" VALUE=\"" + value + "\" " + attributes );   
    return s;
  }


/*
The methods below are the NVR ones.
*/


  /**
   * Returns a text field.
   */
  public static String textFieldNVR( String name, String default_value, HttpServletRequest req, String attributes )
  {
    String value = ( firstTime(req) ? default_value : req.getParameter(name) );
    return textFieldNV( name, value, attributes ); 
  }  


  /**
   * Returns a menu.
   */
  public static String menuNVR( String name, Vector v, Object default_value, HttpServletRequest req, String attributes )
  {
    Object value = ( firstTime(req) ? default_value : req.getParameter(name) );
    return menuN( name, options(v,value), attributes ); 
  }  

 
  /**
   * Returns a menu.
   */
  public static String menuNVR( String name, int min, int max, int step, int default_value, HttpServletRequest req, String attributes )
  {
    int value = ( firstTime(req) ? default_value : (new Integer(req.getParameter(name))).intValue() );
    return menuN( name, options(min,max,step,value), attributes ); 
  }  


  /**
   * Returns a check box. If the request contains a parameter for this check
   * box's name, and that parameter is equal to the value, checks it. If
   * the request contains no parameter, uses the default checking.
   */
  public static String checkBoxNVR( String name, String value, boolean default_checked, HttpServletRequest req, String attributes )
  {
    if ( firstTime(req) ) 
      return checkBoxNV( name, value, attributes, default_checked );
    else {
      String last_value = req.getParameter(name);
      return checkBoxNV( name, value, attributes, last_value != null && last_value.equals(value) );
    }
  }  
  

  /**
   * Returns a radio button. If the request contains a parameter for this check
   * box's name, and that parameter is equal to the value, checks it. If
   * the request contains no parameter, uses the default checking.
   */
  public static String radioButtonNVR( String name, String value, boolean default_checked, HttpServletRequest req, String attributes )
  {
    if ( firstTime(req) ) 
      return radioButtonNV( name, value, attributes, default_checked );
    else {
      String last_value = req.getParameter(name);
      return radioButtonNV( name, value, attributes, last_value != null && last_value.equals(value) );
    }
  }  
  

  /* Returns true if the URL requested has no parameters.
     We use this in the NVR methods to tell whether field values should come from the parameters,
     or from a default supplied by the caller.
  */
  private static boolean firstTime( HttpServletRequest req )
  { 
    return !( req.getParameterNames().hasMoreElements() );
  }
}


