/* AsciiObjectOutputStream.java */


/* 
A stream that allows us to write serialized objects as parsable ASCII.
This is taken from 
  Java Object Serialization in parsable ASCII format
  Enno Davids, Metva P/L.

  AUUG-Vic/CAUUG: Summer Chapter Technical Conference '98

  http://www.metva.com.au/users/enno/papers/java_ser.html
*/


package kb7.io;

import java.io.*;
import java.util.*;
import java.lang.reflect.*;
import kb7.general.Bug;
 

/**
 * A stream that allows us to write serialized objects.
 */
public class AsciiObjectOutputStream extends PrintWriter
{
  final static int	debug = 0;
  // do we want extra debug output? 0 is off, >0 increases verbosity.

  protected int		indent;		
  // cosmetic whitespace to add

  private	Object		instance;

  private FieldOutputFunction output_function = null;
  // If this is non-null, applies it to every field before
  // converting it to string. If it returns a non-null result,
  // uses that instead of the normal conversion.

  /**
   * create an AsciiObjectOutputStream
   *
   * @param out	the output stream we are operating on.
   */
  public AsciiObjectOutputStream(
	PrintWriter	out
  ) 
  {
	// Hand the stream to PrintWriter to deal with.
	super(out);
  
	// We have a bit of state ourselves which we want handled.
	this.indent = 0;
	this.instance = this;
  }
  

  /**
   * create an AsciiObjectOutputStream
   *
   * @param out	the output stream we are operating on.
   * @param fn a field-output function.
   */
  public AsciiObjectOutputStream(
	PrintWriter	out,
        FieldOutputFunction	fn
  ) 
  {
	this( out );
	this.output_function = fn;
  }



  /**
   * write an object to the stream
   *
   * @param obj	the object to be serialized.
   */
  public synchronized void writeAsciiObject(
	Object	obj
  ) throws IOException
  {
	Class	cl = null;
	int	mod;
  
	try {
		// If we got nothing to write then we have choice of
		//   actions. For now we will simply do nothing. Long
		//   term we may wish to do something to ensure we get
		//   back a null pointer when we read.
		if (obj == null) {
			if (debug > 0)
				System.err.println("tried writing NULL object");
			// throw new IOException("tried writing NULL object");
			return;
		}
  
		if (debug > 0)
			System.err.println("writeAsciiObject("+obj+")");
  
		// We're trying to be flexible here. There's a limit
		//   though to what makes sense. Mostly obj should
		//   be an instance of something.
		if (obj instanceof Class)
			cl = (Class)obj;
		else
			cl = obj.getClass();
  
		// start writing the class itself. If we're calling
		//   ourselves the class name has already been emitted.
		if (indent < 1)
			println("class " +
				obj.getClass().getName() +
				" = {");
  
		// recurse up the chain of superclasses and emit
		//   their fields...
		write_superclass(obj, cl.getSuperclass());
  
		// ... and iterate along the fields declared here.
		write_fields(obj, obj);
  
		// end marker.
		if (indent < 1)
			println("}\n");
	}
	catch (Exception e) {
		throw Bug.Bug( "Error writing object: ", e );
	}
  }


  /**
   * write an object's superclass fields to the stream. Note to output
   *   fields in order we traverse the superclasses depth first. At
   *   each superclass we emit the locally declared fields.
   *
   * @param obj	the object to be serialized.
   * @param cl	the (super)class type we are dealing with.
   */
  void write_superclass(
	Object	obj,
	Class	cl
  ) throws Exception
  {
	if (cl != null) {
		write_superclass(obj, cl.getSuperclass());
		write_fields(obj, cl);
	}
  }


  /**
   * write an object's fields to the stream. The fields we will attempt
   *   to write are all the fields declared in this class. This breaks
   *   badly if we encounter fields which are not public in particular
   *   as we will get IllegalAccesException when we try to fill them
   *   in. C'est La Vie. JDK1.1 gets around this by having a native
   *   method bypass all that inconvenient checking for it.
   *
   * @param obj		the object to be serialized.
   * @param classobj	the class type we are dealing with.
   */
  void write_fields(
	Object	obj,
	Object	classobj
  ) throws Exception {
	Class		cl;
	Field		fields[];
	StringBuffer	indentbuf = null;
	String		type = null;

	// To make the our output more human readable we prepend
	//   whitespace in the usual manner to show nesting of
	//   objects and their data.
	indent++;
	indentbuf = new StringBuffer();
	for (int i=0 ; i<indent ; i++)
		indentbuf.append("\t");

	// once again we attempt to flexible in what we will deal with.
	if (classobj instanceof Class)
		cl = (Class)classobj;
	else
		cl = classobj.getClass();

	// detect when we've reached out limits. This is particularly
	//   the case when we're traversing the superclass chain.
	if ((cl == null) || (cl.isInstance(new Object()))) {
		indent--;
		return;
	}

	// we emit the class we're dealing with as a useful diagnostic
	println(indentbuf + "// " + classobj);

	// Now we process each field in turn.
	fields = cl.getDeclaredFields();
	for (int i=0 ; i<fields.length ; i++) {
		Class	ctype = fields[i].getType();
		int	mod;

		mod = fields[i].getModifiers();
		if (Modifier.isStatic(mod))
			continue;

		// lowest level debug prints each field as we see it.
		if (debug > 0)
			System.err.println(
				"field " + i + ":   " + fields[i]);

                Object contents = fields[i].get(obj);
                if ( debug > 0 )
                        System.err.println( " " + contents );

		print(indentbuf +
			getTypeName(ctype) + " " +
			fields[i].getName() +
			(ctype.isArray() ?
				"[" + ( contents == null ? "null" : Array.getLength(contents)+"" ) + "]" :
				"") +
			" =");

                if ( contents == null )
                        println( " null" );
                else if (ctype.isPrimitive()) {
			if (
				ctype.equals(Boolean.TYPE) ||
				ctype.equals(Character.TYPE) ||
				ctype.equals(Byte.TYPE) ||
				ctype.equals(Short.TYPE) ||
				ctype.equals(Integer.TYPE) ||
				ctype.equals(Long.TYPE) ||
				ctype.equals(Float.TYPE) ||
				ctype.equals(Double.TYPE) ||
				ctype.equals(Void.TYPE)
			) {
				handle_instance_field(
					contents );
			} else {
				throw new IOException("unknown primitive type");
			}
		} else if (ctype.isArray()) {
			int		j, max;

			max = Array.getLength( contents );
			println(" {");
			for (j=0 ; j<max ; j++) {
				handle_array_field(
					j,
					Array.get( contents, j ),
					ctype);
			}
			println(indentbuf + "}");
		} else {
			if (ctype.isInstance(new String())) {
				handle_instance_field(
					contents );
			} else {
				String own_output = null;
				if ( this.output_function != null ) 
					own_output = this.output_function.output( cl, fields[i].getName(), contents );
				if ( own_output != null ) {
					println( " {");
					println( own_output );
					println( "}" );
				}
				else {
					if (contents == null)
						println(" null");
					else {
						println(" {");
						handle_instance_field(contents);
						println(indentbuf + "}");
					}
				}
			}
		}
	}

	indent--;
  }


  void handle_instance_field(
	Object	obj
  ) throws Exception
  {
	// print("[[[ " + obj + " ]]]  ");
  
	if (obj == null)
		println(" null");
  
	else if (
		(obj instanceof Boolean) ||
		(obj instanceof Character) ||
		(obj instanceof Byte) ||
		(obj instanceof Short) ||
		(obj instanceof Integer) ||
		(obj instanceof Long) ||
		(obj instanceof Float) ||
		(obj instanceof Double)
	) {
		println(" " + obj);
  
	} else if (obj instanceof String) {
		print(" ");
		writeString((String)obj);
		println("");
  
	} else {
		writeAsciiObject(obj);
	}
}


  void handle_array_field(
	int	j,
	Object	obj,
	Class	ctype
  ) throws Exception
  {
	StringBuffer	newindent = new StringBuffer();
  
	for (int i=0 ; i<indent+1 ; i++)
		newindent.append("\t");
  
	if (obj == null)
		println(newindent +
			getTypeName(ctype) + " " +
			j + " = null");
	else if (
		(obj instanceof Boolean) ||
		(obj instanceof Character) ||
		(obj instanceof Byte)	||
		(obj instanceof Short)	||
		(obj instanceof Integer) ||
		(obj instanceof Long)	||
		(obj instanceof Float)	||
		(obj instanceof Double)	||
		(obj instanceof String)
	) {
		print(newindent +
			getTypeName(ctype) + " " +
			j + " =");
		handle_instance_field(obj);
	} else {
		println(newindent +
			getTypeName(ctype) + " " +
			j + " = {");
		// print(newindent + "\t");
		indent++;
		handle_instance_field(obj);
		indent--;
		println(newindent + "}");
	}
  }


  String getTypeName(
	Class	ctype
  ) throws IOException
  {
	String	type = null;
  
	if (ctype.isPrimitive()) {
		type = ctype.getName();
	} else if (ctype.isArray()) {
		int	depth = 0;
		Class	atype = ctype;
  
		while(atype.isArray()) {
			depth++;
			atype = atype.getComponentType();
			type = getTypeName(atype);
		}
		if (depth > 1)
			throw new IOException(
				"more than 1 array dimension found [" +
				depth + "]");
	} else {
		type = (ctype.isInstance(new String())) ?
				"String" :
				"class " + ctype.getName();
	}
  
	return type;
  }


  /**
   * write a String to the stream.
   *
   * @param s	the string to be serialized.
   */
  void writeString(
	String	s)
  throws IOException
  {
	if (s == null) {
		print("null");
	} else if (s.length() == 0) {
		print("\"\"");
	} else {
		print("\"" + s + "\"");
	}
  }
}

