/* Lister.java */


package kb7.lister;
import java.io.PrintWriter;
import java.util.Stack;
import java.util.Vector;
import kb7.structures.Map;


/**
 * This class implements a data structure that records lines from
 * the source program and can annotate them with error reports,
 * listing the whole thing to an output stream. Listings can be
 * requested with or without line numbers, and you can ask for source
 * lines only or annotations only.
 * <P> 
 * Note that all line numbers passed to the public methods start at 1.
 * Line number 0 may be passed to addAnnotation for annotations which
 * are to appear before any of the source. Line number END may be
 * passed for general annotations about the entire program, which are
 * to appear after all the others.
 * <P>
 * Annotations are represented by instances of class Annotation.
 * An important point is that we may often want an annotation to span
 * more than one line. For example, a syntax error message might 
 * have the error message on one line, the current token on the next,
 * and the expected tokens on the third. If the listing is being given
 * with line numbers, we would like to indent each of the lines making
 * up the annotation so that they match the indentation enforced by
 * prepending the line numbers. This suggests that the best way to
 * return an annotation's printable representation is as a list of
 * lines, not as one string. Consequently, the main method for getting
 * an Annotation's printable representation is toStrings(), which
 * returns a Vector, and not toString().
 */
public class Lister
{ 
  public final static int SOURCE=1;
  public final static int NO_SOURCE=0;
  public final static int ANNOTATIONS=2;
  public final static int NO_ANNOTATIONS=0;
  public final static int LINE_NUMBERS=4;
  public final static int NO_LINE_NUMBERS=0;
  // Public constants controlling whether the -list- method emits the
  // source, annotations, and line numbers.

  public final static int END=-2;
  // As a line-number argument, this represents an
  // annotation which is a general comment about the entire soure,
  // rather than one tied to a particular line. It should appear
  // at the end in a listing, but never with a line number.

  
  private Stack files = new Stack();
  // The stack of included files. The top is the file
  // currently being listed to.

  
  public Lister()
  {
    this.files.push( new FileBody("") );
  }


  /** 
   * This is a test method.
   */
  public static void main( String[] argv )
  {
    Lister l = new Lister();

    PrintWriter out = new PrintWriter( System.out, true );

    l.addAnnotation( "Error at start" );
    l.addLine( "Line 1" );
    l.addAnnotation( "Error after 1" );
    l.addLine( "Line 2" );
    l.addLine( "Line 3" );
    l.addLine( "Line 4" );
    l.addAnnotation( 4, "Error 1 after 4" );
    l.addAnnotation( 3, "Error after 3" );
    l.addAnnotation( 4, "Error 2 after 4" );
    l.addAnnotation( 5, "Error 1 after 5" );
    l.pushFile( "f" );
    l.addLine( "Included line f1" );
    l.addLine( "Included line f2" );
    l.addAnnotation( "Error after included line f1" );
    l.pushFile( "g" );
    l.addLine( "Included line fg1" );
    l.addLine( "Included line fg2" );
    l.addAnnotation( "Error after included line fg1" );
    l.popFile();
    l.pushFile( "h" );
    l.addLine( "Included line fh1" );
    l.addLine( "Included line fh2" );
    l.addAnnotation( "Error after included line fh1" );
    l.popFile();
    l.addLine( "Included line f3" );
    l.addAnnotation( "Error after included line f3" );
    l.addLine( "Included line f4" );
    l.popFile();
    l.addAnnotation( END, "Error at end" );

    System.out.println( "Default listing" );
    l.list( out );
    System.out.println();

    System.out.println( "Listing with source, no annotations, and line numbers" );
    l.list( out, SOURCE | NO_ANNOTATIONS | LINE_NUMBERS );
    System.out.println();

    System.out.println( "Listing with no source, annotations, and line numbers" );
    l.list( out, NO_SOURCE | ANNOTATIONS | LINE_NUMBERS );
    System.out.println();

    System.out.println( "Listing with source, no annotations, and no line numbers" );
    l.list( out, SOURCE | NO_ANNOTATIONS | NO_LINE_NUMBERS );
    System.out.println();

    System.out.println( "Listing with no source, annotations, and no line numbers" );
    l.list( out, NO_SOURCE | ANNOTATIONS | NO_LINE_NUMBERS );
  }


  /** 
   * Adds -line- to the listing at the current file.
   */
  public void addLine( String line )
  {
    getCurrentFile().addLine( line );
  }


  /** 
   * Adds -annotation- after the line with index line_number. 
   * Annotations are emitted in the order they were added.
   */
  public void addAnnotation( int line_number, Annotation annotation )
  {
    getCurrentFile().addAnnotation( line_number, annotation );
  }

  /** 
   * As above, but adds a String.
   */
  public void addAnnotation( int line_number, String annotation )
  {
    addAnnotation( line_number, new StringAnnotation(annotation) );
  }

 
 /** 
   * Adds -annotation- after the current line.
   */
  public void addAnnotation( Annotation annotation )
  {
    getCurrentFile().addAnnotation( annotation );
  }

  /** 
   * As above, but adds a String.
   */
  public void addAnnotation( String annotation )
  {
    addAnnotation( new StringAnnotation(annotation) );
  }


  /**
   * Inserts a new FileBody after the current line.
   */
  public void pushFile( String filename )
  {
    FileBody f = getCurrentFile().addFile( filename );
    this.files.push( f );
  }

  public void popFile()
  {
    this.files.pop();
  }


  /* Returns the outermost file.
  */
  private FileBody getOuterFile()
  {
    return (FileBody)this.files.elementAt(0);
  }


  /* Returns the current file, i.e. that on top of
     the stack.
  */
  protected FileBody getCurrentFile()
  {
    return (FileBody)this.files.peek();
  }


  /** 
   * List to the output stream. -options controls the listing format.
   */
  public void list( PrintWriter out, int options )
  { 
    getOuterFile().list( out, options ); 
  }

  /** 
   * List with default options.
   */
  public void list( PrintWriter out )
  {
    getOuterFile().list( out );
  }
}


/* This class represents one file, either the main one or
   an included one.
*/
class FileBody
{
  private final static String eol = System.getProperty( "line.separator", "\n" );
  // The end-of-line character.

  private String filename;
  // Name of included file that this represents.

  private Vector lines = new Vector();
  // List of lines. 
  private int last_line = -1;
  // Index in lines of line last added.
  // Note that this is 0, not 1, for the first line.

  private int final_line = -1;
  // Maximum line number used in annotations as well as lines.
  // Included in case any annotations refer to lines beyond
  // those added from the source.

  private Map annotations = new Map();
  // Maps line numbers to annotations.

  private Map lines_to_includes = new Map();
  // Maps line numbers to included files, represented
  // as FileBody.

  private Map filenames_to_includes = new Map();
  // Maps filenames to the same included files.


  public FileBody( String filename )
  {
    this.filename = filename;
  }


  /** 
   * Adds -line- to the listing.
   */
  public void addLine( String line )
  {
    this.lines.addElement( line );
    this.last_line++;
    this.final_line = Math.max( this.last_line, this.final_line );
  }


  /** 
   * Adds -annotation- after the line with index line_number. 
   * Annotations are emitted in the order they were added.
   */
  public void addAnnotation( int line_number, Annotation annotation )
  {
    Integer Line_number = new Integer(line_number-1);
    Vector line_annotations = (Vector)this.annotations.get(Line_number);
    if ( line_annotations == null ) 
      line_annotations = new Vector();
    line_annotations.addElement( annotation ); 
    this.annotations.put( Line_number, line_annotations );
    if ( line_number!=Lister.END )
      this.final_line = Math.max( line_number-1, this.final_line );
    // END is a special case, so must be tested for.
  }
 
 /** 
   * Adds -annotation- after the current line.
   */
  public void addAnnotation( Annotation annotation )
  {
    addAnnotation( this.last_line+1, annotation );
  }


  /**
   * Creates a new FileBody and inserts it after the
   * current line. 
   * Returns it in case we need it for something else.
   */
  public FileBody addFile( String filename )
  {
    Integer Line_number = new Integer(this.last_line);
    FileBody included = new FileBody( filename );
    this.lines_to_includes.put( Line_number, included );
    this.filenames_to_includes.put( filename, included );
    return included;
  }


  /** 
   * Return a string containing the listing. -options controls the listing format.
   */
  public String list( int options )
  { 
    String result = "";
    if ( (options & Lister.ANNOTATIONS) != 0 ) 
     result += listAnnotations( -1, options );
    for ( int line_no=0; line_no<this.lines.size(); line_no++ ) 
      result += listLineAndAnnotations( line_no, options );
    if ( (options & Lister.ANNOTATIONS) != 0 ) { 
      for ( int line_no=this.lines.size(); line_no<=this.final_line; line_no++ ) 
        result += listAnnotations( line_no, options );
      result += listEndAnnotations( options );
    }
    return result;
  }

  /** 
   * List to the output stream. -options controls the listing format.
   */
  public void list( PrintWriter out, int options )
  { 
    out.println( list(options) );
  }

  /** 
   * List with default options.
   */
  public String list()
  {
    return list( (Lister.SOURCE | Lister.ANNOTATIONS | Lister.LINE_NUMBERS) );
  }

  /** 
   * List with default options.
   */
  public void list( PrintWriter out )
  {
    list( out, (Lister.SOURCE | Lister.ANNOTATIONS | Lister.LINE_NUMBERS) );
  }


  protected String listLineAndAnnotations( int line_no, int options )
  {
    String result = "";
    if ( (options & Lister.SOURCE) != 0 ) 
      result += listLine( line_no, options );
    if ( (options & Lister.ANNOTATIONS) != 0 ) 
      result += listAnnotations( line_no, options );
    result += listIncludes( line_no, options );
    return result;
  }


  protected String listEndAnnotations( int options )
  {
    return listAnnotations( Lister.END-1, options );
    // Because addAnnotation will have subtracted 1 from END before
    // storing the annotation.
  }


  /* Gets the line at line_no and returns it, adding a line number or indentation
     if necessary.
  */
  private final String listLine( int line_no, int options )
  {
    String line = (String)this.lines.elementAt(line_no);
    line = format( line, line_no, (options & Lister.LINE_NUMBERS) != 0, true );
    return line + eol;
  }


  /* Gets the annotations for line_no and returns them, adding a line number or
     indentation if necessary.
  */
  private final String listAnnotations( int line_no, int options )
  {
    String result = "";

    Vector line_annotations = (Vector)this.annotations.get( new Integer(line_no) );

    if ( line_annotations != null ) {
      for ( int i=0; i<line_annotations.size(); i++ ) {
        Annotation annotation = (Annotation)line_annotations.elementAt(i);
        Vector lines = annotation.toStrings();
        for ( int j=0; j<lines.size(); j++ ) {
          boolean add_line_number = 
            ( (options & Lister.SOURCE) == 0 ) & 
            (i==0) & (j==0) &
            (line_no > -1) & (line_no != Lister.END);
          String line = (String)lines.elementAt(j);
          line = format( line, line_no, (options & Lister.LINE_NUMBERS) != 0, add_line_number );
          // If source is not being listed, prepend the line number
          // to the first annotation; align others consistently with it.
          // Unless it is zero, meaning that the annotation is appearing before
          // any source, or END, meaning that the annotation is a general comment
          // about the entire source.
          // if source is being listed, do not add the line number (it will be
          // in a line of source above), but do align.
          result += line + eol;   
        }
      }
    }

    return result;
  }


  /* Gets the include for line_no and returns it, if any.
  */
  private final String listIncludes( int line_no, int options )
  {
    String result = "";

    FileBody included = (FileBody)this.lines_to_includes.get( new Integer(line_no) );

    if ( included != null ) {
      result += eol;
      String message = format( "INCLUDE ", line_no, (options & Lister.LINE_NUMBERS) != 0, false );
      result += message + eol;
      result += included.list( options );

      message = format( "END INCLUDE ", line_no, (options & Lister.LINE_NUMBERS) != 0, false );
      result += message + eol;
      result += eol;
    }

    return result;
  }


  /* Adjusts the line being output by prepending either a right-justified
     line number, or an equivalent number of spaces, or nothing.
     The line_numbers argument corresponds to the LINE_NUMBERS option. If
     this is false, we prepend nothing, just returning the original line.
     If it is true, we prepend either the line number (if add_line_number),
     or the same number of spaces. These are to push the line right so that
     it aligns with any lines that do have numbers.
  */
  private final String format( String line, int line_number, 
                               boolean line_numbers, boolean add_line_number 
                       )
  {
    if ( line_numbers ) {
      String prefix;
      if ( add_line_number ) {
        line_number++; // Because it's internal, so based at 0. 
                       // Need to convert to what the caller sees.
        if ( line_number < 10 )
          prefix = "  " + line_number;
        else if ( line_number < 100 )
          prefix = " " + line_number;
        else
          prefix = "" + line_number;
        prefix = prefix + ": ";
      }
      else
        prefix = "     ";
      return prefix + line;
    }
    else
      return line;
  }
}


