/* ListerInputStream.java */


package kb7.lister;

import java.io.FilterInputStream;
import java.io.InputStream;
import java.io.IOException;
import java.io.StringBufferInputStream;
import java.io.PrintWriter;


/**
 * This class implements an input stream which filters its input,
 * recording the current line and adding it to a Lister whenver
 * it is completed.
 * <P>
 * To use it, call the constructor, and then the read methods, 
 * closing the stream when finished. 
 * <P>
 * The stream does not override skip.
 */
public class ListerInputStream extends FilterInputStream
{
  private Lister lister = null;
  // Records the source listing.

  private StringBuffer line = null;
  // The line currently being read.

  private boolean line_flushed = true;
  // True once the current line has been added to the Listing. 

  private boolean previous_char_is_CR = false;
  private boolean previous_char_is_LF = false;
  // For detecting line ends. True if the previous character was
  // a carriage return or line feed respectively.


  /** 
   * This is a test method.
   */
  public static void main( String[] argv )
  {
    try {
      PrintWriter out = new PrintWriter( System.out, true );
      Lister l = new Lister();
      String eol = System.getProperty( "line.separator", "\n" );
      String source = "Line 1" + eol + "Second line" + eol + "Final line" + eol;
      StringBufferInputStream sin = new StringBufferInputStream( source );
      ListerInputStream lin = new ListerInputStream( sin, l );
      int the_byte;
      while ( ( the_byte = lin.read() ) != -1 ) 
        ;
      out.println( "Before close" );
      l.list( out );
      out.println();
      out.println( "After close" );
      l.list( out );
    }
    catch ( IOException e ) {
      System.out.println( e );
    }
  }


  /** 
   * Constructs a ListerInputStream from an InputStream and a
   * Listing.
   */
  public ListerInputStream( InputStream in, Lister lister )
  {
    super(in);
    this.lister = lister;
    startNewLine(); 
  }


  /** 
   * Overrides the underlying stream's read(,,) method, by updating
   * with the characters (if any) read.
   */
  public final int read( byte b[], int off, int len ) throws IOException
  {
    int bytes_read = this.in.read( b, off, len );
    if ( bytes_read != -1 ) 
      for ( int i = 0; i<bytes_read; i++ ) {
        updateLine( (char)b[i+off] );
      }
    else
      endLine();
    return bytes_read;
  }


  /** 
   * Overrides the underlying stream's read() method, by updating
   * with the character (if any) read.
   */
  public final int read() throws IOException
  {
    int the_byte = this.in.read();
    if ( the_byte != -1 )
      updateLine( (char)the_byte );
    else
      endLine();
    return the_byte;
  }


  /** 
   * Overrides the underlying stream's close() method, by
   * ensuring that the final line has been added to the Lister.  
   * It should be unnecessary to check this here, since the read methods
   * check for end-of-input, but there's no harm in doing so.
   */
  public final void close() throws IOException
  {
    endLine();
  }


  /* Prepares for a new line.
  */
  private final void startNewLine()
  {
    this.line = new StringBuffer();
    this.line_flushed = false;
  }


  /* Checks whether the current line has already been added to
     the Listing. If not, adds it and prepares for a new line.
  */
  private final void endLine()
  {
    if ( ! this.line_flushed ) {
      this.lister.addLine( this.line.toString() );
      startNewLine();
    }
  }


  /* Either adds c to the current line, or if it appears to end the 
     line, calls endLine().
  */
  private final void updateLine( char c )
  {
    /* This piece of code was adapted from something
       similar which occurs in JavaCC's ASCII_CharStream.
    */
    if ( previous_char_is_LF ) {
      previous_char_is_LF = false;
      endLine();
    }
    else if ( previous_char_is_CR ) {
      previous_char_is_CR = false;
      if ( c == '\n' ) 
        previous_char_is_LF = true;
      else
        endLine();
    }

    switch (c)
    {
      case '\r' :
        previous_char_is_CR = true;
        break;
      case '\n' :
        previous_char_is_LF = true;
        break;
      default :
        this.line.append( c );
    }
  }
}
