/* ErrorLister.java */


package kb7.lister;

import java.util.Vector;
import kb7.strings.Strings;


/**
 * This class extends our standard Lister with a method for
 * adding SourceErrors/Warnings/Notes as annotations, 
 * and methods for recording and returning the number of these 
 * so far added. These methods also print the messages to
 * System.out, ensuring that the user will see it even if the
 * listing file doesn't appear, e.g. because the compiler
 * crashes before closing it.
 */
public class ErrorLister extends Lister
{
  private int error_count = 0;
  private int warning_count = 0;
  private int note_count = 0;
  // Number of items added so far.


  /**
   * Adds a SourceError, incrementing the error count.
   */
  public void addError( SourcePosition sp, SourceError error )
  {
    printAnnotation( sp.getLineNumber(), markColumnAndSplit(sp.getColumnNumber(),error) );
    addAnnotation( sp.getLineNumber(), markColumnAndSplit(sp.getColumnNumber(),error) );
    this.error_count++;
  }


  /**
   * Adds a SourceError to the end of the program,
   * incrementing the error count.
   */
  public void addErrorAtEnd( SourceError error )
  {
    printAnnotation( Lister.END, split(error) );
    addAnnotation( Lister.END, split(error) );
    this.error_count++;
  }


  /**
   * Adds a SourceWarning, incrementing the warning count.
   */
  public void addWarning( SourcePosition sp, SourceWarning warning )
  {
    printAnnotation( sp.getLineNumber(), markColumnAndSplit(sp.getColumnNumber(),warning) );
    addAnnotation( sp.getLineNumber(), markColumnAndSplit(sp.getColumnNumber(),warning) );
    this.warning_count++;
  }

 
  /**
   * Adds a SourceWarning to the end of the program,
   * incrementing the warning count.
   */
  public void addWarningAtEnd( SourceWarning warning )
  {
    printAnnotation( Lister.END, split(warning) );
    addAnnotation( Lister.END, split(warning) );
    this.warning_count++;
  }


  /**
   * Adds a SourceNote, incrementing the note count.
   */
  public void addNote( SourcePosition sp, SourceNote note )
  {
    printAnnotation( sp.getLineNumber(), markColumnAndSplit(sp.getColumnNumber(),note) );
    addAnnotation( sp.getLineNumber(), markColumnAndSplit(sp.getColumnNumber(),note) );
    this.note_count++;
  }

 
  /**
   * Adds a SourceNote to the end of the program,
   * incrementing the note count.
   */
  public void addNoteAtEnd( SourceNote note )
  {
    printAnnotation( Lister.END, split(note) );
    addAnnotation( Lister.END, split(note) );
    this.note_count++;
  }


  /**
   * Adds a blank line to the end of the program.
   */
  public void addBlankLineAtEnd()
  {
    addAnnotation( Lister.END, new StringAnnotation("") );
  }


  /** 
   * Returns the number of errors so far.
   */
  public int getErrorCount()
  {
    return this.error_count; 
  }

  /** 
   * Returns the number of warnings so far.
   */
  public int getWarningCount()
  {
    return this.warning_count; 
  }

  /** 
   * Resets the error and warning counts to zero.
   */
  public void clearCounts()
  {
    this.error_count = 0;
    this.warning_count = 0;
  }


  /* This method displays the up-arrow first, then splits the
     lines into chunks of 72 or less, then displays the blank line.
  */
  private VectorAnnotation markColumnAndSplit( int column, Annotation error )
  {
    Vector message = error.toStrings();
    Vector message2 = new Vector();
    String s = "";
    for ( int i=1; i<column; i++ )
      s += " ";
    s += "^";  
    message2.addElement( s );
    for ( int j=0; j<message.size(); j++ ) {
      Vector split = Strings.split( (String)message.elementAt(j), 72 );
      for ( int k=0; k<split.size(); k++ )
        message2.addElement( (String)split.elementAt(k) );
    }
    message2.addElement( "" );
    return new VectorAnnotation( message2 );    
  }


  /* This method splits the lines into chunks of 72 or less.
     This is for wrapping reports to be displayed at the end of
     the program. It's like markColumnAndSplit, but does not
     insert the line with the arrow.
  */
  private VectorAnnotation split( Annotation error )
  {
    Vector message = error.toStrings();
    Vector message2 = new Vector();
    for ( int j=0; j<message.size(); j++ ) {
      Vector split = Strings.split( (String)message.elementAt(j), 72 );
      for ( int k=0; k<split.size(); k++ )
        message2.addElement( (String)split.elementAt(k) );
    }
    return new VectorAnnotation( message2 );    
  }


  /* Prints an annotation to standard output. 
  */
  private void printAnnotation( int line_number, Annotation ann )
  {
    java.io.PrintWriter pw = new java.io.PrintWriter( System.out, true );
    if ( line_number != Lister.END )
      super.listEndAnnotations( pw, (Lister.SOURCE | Lister.ANNOTATIONS | Lister.LINE_NUMBERS) );
    else
      super.listLineAndAnnotations( pw, line_number, (Lister.SOURCE | Lister.ANNOTATIONS | Lister.LINE_NUMBERS) );
  }
}
