/* Persistence.java */


package kb7.persistence;

import java.io.*;
import kb7.general.Bug;


/**
 * This class provides write and read methods for passing objects
 * to and from files. It uses Java's object serialization methods.
 */
public class Persistence
{

  private static class TestObject implements Serializable
  {
    int i;
    Float f;
    java.util.Vector v = new java.util.Vector();
    java.util.Hashtable h = new java.util.Hashtable();

    public String toString()
    {
      return "[TestObject " + this.i + " " + this.f +
                        " " + this.v + " " + this.h +
             "]"; 
    }
  }
  
  public static void main( String argv[] )
  {
    TestObject o = new TestObject();
    o.i = 123;
    o.f = new Float(4.56);
    o.v.addElement( new Integer(7) );
    o.v.addElement( "8" );
    o.h.put( "9", "10" );

    try {
      byte[] bytes = tobyteArray(o);
      TestObject copy = (TestObject)frombyteArray(bytes);
    
      System.out.println(o);
      System.out.println(copy);
    }
    catch ( Throwable e ) {
      System.out.println( e );
    }
  }


  /**
   * Writes o to filename.
   */
  public final static void write( Serializable o, String filename )
                           throws IOException
  { 
    FileOutputStream ostream = new FileOutputStream(filename);
    ObjectOutputStream out = new ObjectOutputStream(ostream);
    out.writeObject(o);
    out.flush();
    out.close();
  }


  /**
   * Reads an object from filename.
   */
  public final static Serializable read( String filename )
                                   throws IOException, ClassNotFoundException
  { 
    FileInputStream istream = new FileInputStream(filename);
    ObjectInputStream in = new ObjectInputStream(istream);
    Serializable s = (Serializable)in.readObject();
    in.close();
    return s;
  }


  /**
   * A utility method to ensure that the filename where the 
   * data will be stored exists, with any directories in its path,
   * because we may not have used it before. 
   * We use the separator "/" or "\" to separate directories.
   * The filename returned will have this replaced by
   * whatever is appropriate for the machine.
   * After doing this, we then call mkdirs() to make all the
   * directories.
   */
  public static String ensureExists( String full_filename )
  {
    full_filename = full_filename.replace( '/', File.separatorChar );
    full_filename = full_filename.replace( '\\', File.separatorChar );
    File data_file = new File( full_filename );
    File data_dir = new File( data_file.getParent() ); 
    if ( !data_dir.exists() )
      data_dir.mkdirs();
    return full_filename;
  }


  /**
   * Converts o to a string. The string is the hex encoding
   * of the result of serializing, since without converting to
   * hex, we might get unprintable characters.
   */
  public final static String toString( Serializable o )
                             throws IOException
  { 
    byte[] bytes = tobyteArray(o);
    String result = "";
    for ( int i=0; i<bytes.length; i++ ) {
      byte b = bytes[i];
      String byte_as_string = Integer.toHexString( b+128 );
      // Add 128 because bytes run from -128 to 127.
      if ( byte_as_string.length() == 1 )
        byte_as_string = "0" + byte_as_string;
      result += byte_as_string;
    }
    return result;
  }


  public final static byte[] tobyteArray( Serializable o )
                             throws IOException
  { 
    StringBuffer sb = new StringBuffer();
    ByteArrayOutputStream ostream = new ByteArrayOutputStream();
    ObjectOutputStream out = new ObjectOutputStream(ostream);
    out.writeObject(o);
    out.flush();
    out.close();
    byte[] bytes = ostream.toByteArray();
    return bytes;
  }


  /**
   * Reads an object from a string.
   */
  public final static Serializable fromString( String str )
                                   throws IOException, ClassNotFoundException
  { 
    byte[] bytes = new byte[ str.length()/2 ];
    for ( int i=0; i<(str.length()/2)-1; i+=2 ) {
      String byte_as_hex = str.substring( i, i+2 );
System.out.println(byte_as_hex);
      int byte_as_int = Integer.parseInt( byte_as_hex, 16 );
      byte b = (byte)( byte_as_int - 128 );
    }
    Serializable s = frombyteArray(bytes);
    return s;
  }


  /**
   * Reads an object from a byte array.
   */
  public final static Serializable frombyteArray( byte[] bytes )
                                   throws IOException, ClassNotFoundException
  { 
    ByteArrayInputStream istream = new ByteArrayInputStream(bytes);
    ObjectInputStream in = new ObjectInputStream(istream);
    Serializable s = (Serializable)in.readObject();
    in.close();
    return s;
  }


  public final static Serializable copy( Serializable o )
  {
    try {
      return frombyteArray( tobyteArray(o) );
    }
    catch ( Throwable e ) {
      throw new Bug( "Error copying object: " + e );
    }
  }
}
