/* PersistentMap.java */


package kb7.persistence;

import java.io.File;
import java.util.Enumeration;
import java.util.Vector;
import kb7.general.Bug;
import kb7.structures.Map;


/**
 * This class represents a persistent map, i.e. one that can be saved 
 * to, and restored from, a file.
 * 
 * It includes methods for saving to, and restoring from, the files
 * we use to implement our database. In general, methods that
 * change the state come in two versions: one that saves the change
 * to the database, and one that doesn't.
 * 
 * Each instance is created in an 'unloaded' state, meaning that it
 * does not contain the data implementing the map. Calling load()
 * switches into a loaded state, in which the map can be used, via
 * the same methods as kb7.structures.Map offers. Calling unload()
 * returns to an unloaded state. 
 *
 * Methods that modify the map save their changes to the database
 * just before returning. They have variants whose names end in "noDB"
 * which don't save. The instance has an internal flag recording 
 * whether there are any unsaved changes: an attempt to unload the
 * instance while there are unsaved changes will throw a Bug.
 *
 * The class's constructor takes a filename, including path and extension.
 * This is the file in which data will be stored, and it is always 
 * associated with the instance. When the instance is first created,
 * if any directories in the path don't exist, they will be created. 
 * The constructor will also try writing to and reading from the file,
 * ensuring that errors will be caught early.
 *
 */
public class PersistentMap 
       implements java.io.Serializable // So we can write out objects that contain it.
{
  private Map map = new Map();
  // The map which we use.

  private boolean loaded = false;
  // Whether the state is loaded or unloaded.

  private boolean unsaved_changes = false;
  // Whether all changes have been saved. Only meaningful
  // in loaded state.

  private String filename = null;
  // The full name of the file in which data is
  // to be stored.

  private boolean tracing = false;
  // If this is true, the instance will trace
  // loads and saves.


  public static void main( String[] argv )
  {
    PersistentMap pm = new PersistentMap( "d:\\kb7/persistence\\test1/test2\\test", true );
    pm.put( "one", "1" );
    pm.put( "two", "2" );
    pm.put( "three", "3" );
    pm.remove( "two" );
    pm.put( "four", "4" );
    System.out.println( pm.toString() );
    pm.unload();

    PersistentMap pm2 = new PersistentMap( "d:\\kb7/persistence\\test1/test2\\test", false );
    pm2.setTracing( true );
    pm2.load();
    pm2.put( "five", "5" );
    pm2.put( "six", "6" );
    System.out.println( pm2.toString() );

    PersistentMap pm3 = new PersistentMap( "d:\\kb7/persistence\\test1/test2\\test", false );
    pm3.load();
    System.out.println( pm3.toString() );
  }


  /**
   * Creates a new persistent map. 
   * If is_new, set loaded to true. This is for the first
   * ever use of the constructor, when we can't load it
   * because there may be no database file to load from.
   */
  public PersistentMap( String filename, boolean is_new )
  {
    if ( this.tracing )
      System.out.println( "Calling PersistentMap( " + filename + "," + is_new + ")" );
    this.filename = Persistence.ensureExists( filename );
    if ( is_new ) {
      this.loaded = true;
      save();
      unload();
      load();
    }
    else
      this.loaded = false;
    this.unsaved_changes = false;    
    if ( this.tracing )
      System.out.println( "Exiting PersistentMap( " + filename + "," + is_new + ")" );
  }


  /**
   * Returns a map read from the database.
   */
  public synchronized void load()
  {    
    if ( this.tracing )
      System.out.println( "Calling PersistentMap.load()" );
    ensureNotLoaded();
    this.map = (Map)Persistence.read( this.filename );   
    if ( this.tracing ) {
      System.out.println( "In PersistentMap.load(): read from " + this.filename + ":" );
      System.out.println( "  " + this.map.toString() );
    }
    this.loaded = true;
    this.unsaved_changes = false;
    if ( this.tracing )
      System.out.println( "Exiting PersistentMap.load()" );
  }


  /**
   * Unloads the map, saving space by setting it to null.
   * It should have been saved first.
   */
  public synchronized void unload()
  {    
    if ( this.tracing )
      System.out.println( "Calling PersistentMap.unload()" );
    ensureLoaded();
    ensureSaved();
    this.map = null;
    this.loaded = false;
    this.unsaved_changes = false;
    if ( this.tracing )
      System.out.println( "Exiting PersistentMap.unload()" );
  }


  /**
   * Saves the map in the database.
   */
  public synchronized void save()
  {
    if ( this.tracing )
      System.out.println( "Calling PersistentMap.save()" );
    ensureLoaded();
    Persistence.write( this.map, this.filename );  
    if ( this.tracing ) {
      System.out.println( "In PersistentMap.save(): written to " + this.filename + ":" );
      System.out.println( "  " + this.map.toString() );
    }
    this.unsaved_changes = false;
    if ( this.tracing )
      System.out.println( "Exiting PersistentMap.save()" );
  }


  /**
   * Adds the key/value pair to the map, and saves to the database.
   */
  public synchronized void put( Object key, Object value )
  { 
    putNoDB(key,value);
    save();
  } 

  /**
   * Adds the key/value pair to the map.
   */
  public synchronized void putNoDB( Object key, Object value )
  { 
    ensureLoaded();
    this.map.put( key, value ); 
    this.unsaved_changes = true;
  }


  /**
   * Deletes the object with this key, and saves to
   * the database.
   */
  public synchronized void remove( Object key )
  { 
    removeNoDB(key);
    save();
  }

  /**
   * Deletes the object with this key.
   */
  public synchronized void removeNoDB( Object key )
  { 
    ensureLoaded();
    this.map.remove(key);
    this.unsaved_changes = true;
  }


  /**
   * Removes all mappings, and saves to the database.
   */
  public synchronized void removeAll()
  {
    removeAllNoDB();
    save();
  }

  /**
   * Removes all mappings.
   */
  public synchronized void removeAllNoDB()
  {
    this.map.removeAll();
    this.unsaved_changes = true;
  }


  /**
   * True if an object with this key exists.
   */
  public synchronized boolean contains( Object key )
  { 
    ensureLoaded();
    return this.map.contains(key);
  }  


  /**
   * Returns the object with this name.
   */
  public synchronized Object get( Object key )
  { 
    ensureLoaded();
    return this.map.get(key);
  }


  /**
   * Returns the size of the domain.
   */
  public int size()
  {
    ensureLoaded();
    return this.map.size();
  }


  /**
   * Returns the domain.
   */
  public Enumeration domainElements()
  {
    ensureLoaded();
    return this.map.domainElements();
  }


  /**
   * Sets tracing on or off.
   */
  public void setTracing( boolean tracing )
  {
    this.tracing = tracing;
  }


  /**
   * Converts to string for debugging.
   */
  public String toString()
  {
    if ( !this.loaded )
      return "[PersistentMap (" + this.filename + ") unloaded ]";
    else
      return "[PersistentMap (" + this.filename + ") " + this.map.toDisplay() + " ]";
  }


  /* Throw a Bug if the data isn't loaded.
  */
  private void ensureLoaded()
  {
    if ( !this.loaded )
      throw new Bug( "Data not loaded" );
  }  


  /* Throw a Bug if the data is loaded.
  */
  private void ensureNotLoaded()
  {
    if ( this.loaded )
      throw new Bug( "Data already loaded" );
  }


  /* Throw a Bug if the data has been changed
     since the last save.
  */
  private void ensureSaved()
  {
    if ( this.unsaved_changes )
      throw new Bug( "Data not saved" );
  }
}
