/* PersistentMap.java */


package sharesim;

import java.io.File;
import java.util.Vector;
import kb7.general.Bug;
import kb7.structures.Map;


/**
 * This class represents a persistent map, i.e. one that can be saved 
 * to, and restored from, a file.
 * 
 * It includes methods for saving to, and restoring from, the files
 * we use to implement our database. In general, methods that
 * change the state come in two versions: one that saves the change
 * to the database, and one that doesn't.
 * 
 * Each instance is created in an 'unloaded' state, meaning that it
 * does not contain the data implementing the map. Calling load()
 * switches into a loaded state, in which the map can be used, via
 * the same methods as kb7.structures.Map offers. Calling unload()
 * returns to an unloaded state. 
 *
 * Methods that modify the map save their changes to the database
 * just before returning. They have variants whose names end in "noDB"
 * which don't save. The instance has an internal flag recording 
 * whether there are any unsaved changes: an attempt to unload the
 * instance while there are unsaved changes will throw a Bug.
 *
 * The class's constructor takes a filename, including path and extension.
 * This is the file in which data will be stored, and it is always 
 * associated with the instance. When the first save is done, if
 * any directories in the path don't exist, they will be created. 
 */
public class PersistentMap 
{
  private Map map = new Map();
  // The map which we use.

  private boolean loaded = false;
  // Whether the state is loaded or unloaded.

  private boolean unsaved_changes = false;
  // Whether all changes have been saved. Only meaningful
  // in loaded state.

  private String filename = null;
  // The full name of the file in which data is
  // to be stored.


  public static void main( String[] argv )
  {
    PersistentMap pm = new PersistentMap( "d:\\kb7/persistence\\test1/test2\\", true );
    pm.put( "one", "1" );
    pm.put( "two", "2" );
    pm.put( "three", "3" );
    pm.remove( "two" );
    pm.put( "four", "4" );
    System.out.println( pm );
    pm.unload();

    PersistentMap pm2 = new PersistentMap( "d:\\kb7/persistence\\test1/test2\\", false );
    pm2.load();
    System.out.println( pm2 );
  }


  /**
   * Creates a new persistent map. 
   * If is_new, set loaded to true. This is for the first
   * ever use of the constructor, when we can't load it
   * because there may be no database file to load from.
   */
  public PersistentMap( String filename, boolean is_new )
  {
    this.filename = ensureExists( filename );
    if ( is_new )
      this.loaded = true;
    else
      this.loaded = false;
    this.unsaved_changes = false;
  }


  /**
   * Returns a map read from the database.
   */
  public synchronized void load()
  {
    ensureNotLoaded();
    this.map = (Map)Persistence.read( this.filename );   
    this.loaded = true;
    this.unsaved_changes = false;
  }


  /**
   * Unloads the map.
   */
  public synchronized void unload()
  {
    ensureLoaded();
    ensureSaved();
    this.map = null;
    this.loaded = false;
    this.unsaved_changes = false;
  }


  /**
   * Saves the map in the database.
   */
  public synchronized void save()
  {
    ensureLoaded();
    Persistence.write( this.map, this.filename );  
    this.unsaved_changes = false;
  }


  /**
   * Adds the key/value pair to the map, and saves to the database.
   */
  public synchronized void put( Object key, Object value )
  { 
    addObjectNoDB(key,value);
    save();
  } 

  /**
   * Adds the key/value pair to the map.
   */
  public synchronized void putNoDB( Object key, Object value )
  { 
    ensureLoaded();
    this.map.put( key, value ); 
    this.unsaved_changes = true;
  }


  /**
   * Deletes the object with this key, and saves to
   * the database.
   */
  public synchronized void remove( Object key )
  { 
    deleteObjectNoDB(key);
    save();
  }

  /**
   * Deletes the object with this key.
   */
  public synchronized void removeNoDB( Object key )
  { 
    ensureLoaded();
    if ( !this.map.contains(name) )
      throw new Bug( "Object not found " + name );
    this.map.remove(key);
    this.unsaved_changes = true;
  }


  /**
   * True if an object with this key exists.
   */
  public synchronized boolean containsObject( Object key )
  { 
    ensureLoaded();
    return this.map.contains(key);
  }  


  /**
   * Returns the object with this name.
   */
  public synchronized Object get( Object key )
  { 
    ensureLoaded();
    return this.map.get(key);
  }


  /* Throw a Bug if the data isn't loaded.
  */
  private void ensureLoaded()
  {
    if ( !this.loaded )
      throw new Bug( "Data not loaded" );
  }  


  /* Throw a Bug if the data is loaded.
  */
  private void ensureNotLoaded()
  {
    if ( this.loaded )
      throw new Bug( "Data already loaded" );
  }


  /* Throw a Bug if the data has been changed
     since the last save.
  */
  private void ensureSaved()
  {
    if ( this.unsaved_changes )
      throw new Bug( "Data not saved" );
  }


  /* The filename is where the data will be stored.
     Ensure that any directories in its path exist,
     because we may not have used it before. 
     We use the separator "/" or "\" to separate directories.
     The filename returned will have this replaced by
     whatever is appropriate for the machine.
  */
  private static String ensureExists( String full_filename )
  {
    full_filename = full_filename.replace( '/', File.separatorChar );
    full_filename = full_filename.replace( '\\', File.separatorChar );
    File f = new File( full_filename );
    if ( !f.exists() )
      f.mkdirs();
//    Vector split = Strings.splitAt( full_filename, File.separator );
  
//  ensureExists( split, root );
    return full_filename;
  }


  /* Method used internally by ensureExists above. 'sofar' is the
     
  */
/*
  private static void ensureExists( Vector filename, String sofar )
  {
    if ( v.size() == 0 )
      throw new Bug( "Empty filename vector: " + filename + " " + sofar );
    else if ( v.size() == 1 )
      return;
    else {
      String seg0 = (String)filename.elementAt(0);
      File f = new File( sofar + seg0 + File.separator );
      if ( !f.exists() )
        f.mkdir();
      filename.removeElementAt(0);
      ensureExists( filename, sofar + seg0 + File.separator );
    }
  }
*/
}
