/* Strings.java */


package kb7.strings;

import java.util.Vector;
import kb7.general.Bug;


/**
 * This class defines various useful string-handling methods.
 */
public final class Strings
{
  public static void main( String argv[] )
  {
    System.out.println( capitaliseFirst("a") );
    System.out.println( capitaliseFirst("abcde") );
    System.out.println( capitaliseFirst("Abcde") );
    System.out.println( capitaliseFirst("1bcde") );

    System.out.println( split("abc de fgh ij klmn opqrst",11) );
    System.out.println( split("abc de fgh ij klmn opqrst",10) );
    System.out.println( split("abc de fgh ij klmn opqrst",9) );
    System.out.println( split("abc de fgh ij klmn opqrst",6) ); 

    System.out.println( splitAt("","/") );
    System.out.println( splitAt("abcd","/") );
    System.out.println( splitAt("///","/") );
    System.out.println( splitAt("///","/\\") );
    System.out.println( splitAt("a12bc1def12ghij1klmno","12") );

    System.out.println( "'"+trimTrailing("")+"'" );
    System.out.println( "'"+trimTrailing("abc")+"'" );
    System.out.println( "'"+trimTrailing("   ")+"'" );
    System.out.println( "'"+trimTrailing("  abc  ")+"'" );

    System.out.println( "'"+removeWhitespace("")+"'" );
    System.out.println( "'"+removeWhitespace("abc")+"'" );
    System.out.println( "'"+removeWhitespace("   ")+"'" );
    System.out.println( "'"+removeWhitespace("  abc  ")+"'" );
    System.out.println( "'"+removeWhitespace("  a b c  ")+"'" );

    System.out.println( matchesPrefix("abcd","abcd") );     // ""
    System.out.println( matchesPrefix("abcd","abce") );     // null
    System.out.println( matchesPrefix("","abce") );         // null
    System.out.println( matchesPrefix("abcd","") );         // null
    System.out.println( matchesPrefix("abcd","ab*") );      // "cd"
    System.out.println( matchesPrefix("abcd","abc*") );     // "d" 
    System.out.println( matchesPrefix("abcd","abcd*") );    // "" 
    System.out.println( matchesPrefix("abcd","abcde*") );   // null
    System.out.println( matchesPrefix("xyz","y*") );        // null

    System.out.println( replace( "", "ab", "cd" ) );
    System.out.println( replace( "abcd", "ab", "xyz" ) );
    System.out.println( replace( "abcd", "cd", "xyz" ) );
    System.out.println( replace( "abc12def1ghi12jkl1mno2pqr", "12", "X" ) ); 
    System.out.println( replace( "abc12def1ghi12jkl1mno2pqr", "12", "" ) );

    System.out.println( take( "12_34", "_" ) );

    System.out.println( drop( "12_34", "_" ) );
  }


  /**
   * Returns s with the first character capitalised, if a letter.
   */
  public static final String capitaliseFirst( String s )
  {
    String s1 = s.substring(0,1).toUpperCase() +
                s.substring(1);
    return s1;
  }


  /**
   * True if c is a letter.
   */
  public static final boolean isLetter( char c )
  {
    return (c>='a' && c<='z') || (c>='A' && c<='Z');
  }


  /**
   * Splits s into a sequence of substrings, each of length
   * max or less. It splits at spaces, and removes those spaces
   * from the result. The method is intended for reformatting
   * error messages and similar so that they do not extend
   * beyond the width of a line.
   */
  public static final Vector split( String s, int max )
  {
    Vector result = new Vector(); 
    s = s.trim();
    while ( s.length() > max ) {
      int i = max;
      while ( i>=0 && s.charAt(i)!=' ' )
        i--;
      while ( i>=0 && s.charAt(i)==' ' ) 
        i--;
      result.addElement( s.substring( 0, i+1 ) );  
      s = s.substring( i+2, s.length() );  
    } 
    result.addElement( s );
    return result;
  }


  /**
   * Splits s into a sequence of substrings at delim.
   * delim is not included in the resulting vector.
   */
  public static final Vector splitAt( String s, String delim )
  {
    Vector result = new Vector();
    while ( true ) {
      int index = s.indexOf( delim );
      if ( index == -1 ) {
        result.addElement( s );
        return result;
      }
      else {
        result.addElement( s.substring(0,index) );
        s = s.substring( index+delim.length() );
      }
    }
  }


  /** 
   * Removes all whitespace from s.
   */
  public static final String removeWhitespace( String s )
  {
    String result = "";
    for ( int i=0; i<s.length(); i++ ) {
      char c = s.charAt(i);
      if ( !Character.isWhitespace( c ) )
        result += c;
    }
    return result;
  }


  /**
   * Trims trailing whitespace from s.
   */
  public static final String trimTrailing( String s )
  {
    if ( s.length() == 0 )
      return s;
    else {
      char c = s.charAt( s.length()-1 );
      if ( Character.isWhitespace(c) )
        return trimTrailing( s.substring( 0, s.length()-1 ) );
      else
        return s;
    }
  }


  /**
   * If 'pattern' does not contain a star, returns null if the two strings
   * are unequal, "" otherwise.
   * If 'pattern' does contain a star, it must be the final character. Returns
   * null if 'name' does not start with the portion of 'pattern' before the star, and the
   * rest of 'name' otherwise.
   * This is useful when searching a list of strings for one that matches
   * one of a set of known prefixes.
   */
  public static String matchesPrefix( String s, String pattern )
  {
    if ( pattern.indexOf("*") == -1 ) {
      if ( pattern.equals(s) ) 
        return "";
      else
        return null;
    }
    else if ( pattern.indexOf("*") != pattern.length()-1 )
      throw new Bug( "* not at end of pattern: " + pattern );
    else {
      String pattern_pre = pattern.substring( 0, pattern.length()-1 );           
      if ( s.startsWith(pattern_pre) )
        return s.substring( pattern.length()-1, s.length() );
      else
        return null;
    }
  }


  /**
   * Replaces all occurrences of old in s by new.
   */
  public static String replace( String s, String old, String neww )
  {
    StringBuffer result = new StringBuffer();
    while ( true ) {
      int pos = s.indexOf( old );
      if ( pos == -1 ) {
        result.append( s );
        return result.toString();
      }
      else {
        String pre = s.substring( 0, pos );
        result.append( pre );
        result.append( neww );
        s = s.substring( pos+old.length(), s.length() ); 
      }
    }
  }


  /**
   * Returns the portion of s upto but not including the
   * first delimiter.
   */  
  public final static String take( String s, String delimiter ) 
  { 
    int delimiter_pos = s.indexOf(delimiter);
    if ( delimiter_pos == -1 )
      throw new Bug( "No delimiter in string: " + s );
    return s.substring( 0, delimiter_pos );
  }


  /**
   * Removes the portion of s upto and including the
   * first delimiter.
   */  
  public final static String drop( String s, String delimiter ) 
  { 
    int delimiter_pos = s.indexOf(delimiter);
    if ( delimiter_pos == -1 )
      throw new Bug( "No delimiter in string: " + s );
    return s.substring( delimiter_pos+1 );
  }
}
