/* Map.java */


package kb7.structures;


import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import java.lang.StringBuffer;


/**
 * This class implements a mapping.
 */
public class Map 
       extends Function
       implements java.io.Serializable          
{
  private static final long serialVersionUID = 1000;


  private Hashtable table = new Hashtable();
  // Stores the mapping.


  /** 
   * Do-nothing constructor.
   */  
  public Map()
  {
  }


  /**
   * Builds a Map whose elements are given by -table-.
   */ 
  public Map( Hashtable table )
  {
    this.table = table;
  }


  /**
   * Returns the hashtable.
   */
  public Hashtable getHashtable()
  {
    return this.table;
  }


  /**
   * Builds a Map whose elements are given by -pairs-, where
   * each pair's second element is the image of the first.
   */ 
  public Map( Vector pairs )
  {
    for ( int i=0; i < pairs.size(); i++ )
      addElement( (Pair)pairs.elementAt(i) );
  }


  /**
   * Adds a mapping from o1 to o2.
   */
  public synchronized void put( Object o1, Object o2 )
  {
    this.table.put( o1, o2 );
  }  

  /**
   * Adds a mapping from integer i1 to o2.
   */
  public synchronized void put( int i1, Object o2 )
  {
    this.table.put( new Integer(i1), o2 );
  }


  /**
   * Adds a mapping from p.getFirst() to p.getSecond().
   */
  public synchronized void addElement( Pair p )
  {
    put( p.getFirst(), p.getSecond() );
  }


  /**
   * Removes the mapping whose domain element is o.
   */
  public synchronized void remove( Object o )
  {
    this.table.remove( o );
  }


  /**
   * Removes all mappings.
   */
  public synchronized void removeAll()
  {
    this.table = new Hashtable();
  }


  /**
   * Returns the image of key.
   */
  public synchronized Object get( Object key )
  {
    return this.table.get( key );
  } 

 /**
   * Returns the image of key cast to String.
   * Just saves casts in cases where most of
   * our codomain is Strings.
   */
  public synchronized String getString( Object key )
  {
    return (String)this.table.get( key );
  } 

  /**
   * Returns the image of the integer key.
   */
  public synchronized Object get( int key )
  {
    return this.table.get( new Integer(key) );
  }


  /**
   * Returns the domain.
   */
  public Enumeration domainElements()
  {
    return this.table.keys();
  }


  /**
   * Returns the domain as a vector.
   */
  public Vector domainElementsAsVector()
  {
    Vector domain = new Vector();
    Enumeration e = this.table.keys();
    while ( e.hasMoreElements() )
      domain.addElement( e.nextElement() );
    return domain;
  }


  /**
   * Returns the codomain.
   */
  public Enumeration codomainElements()
  {
    return this.table.elements();
  }


  /** 
   * If m2 contains a maplet for the same domain element as one
   * in us, uniting will overwrite ours with that.
   */
  public void unite( Map m2 )
  {
    Enumeration m2_domain_elements = m2.domainElements();
    while ( m2_domain_elements.hasMoreElements() ) {
      Object m2_x = m2_domain_elements.nextElement();
      put( m2_x, m2.get(m2_x) );
    }
  }


  /**
   * Return the inverse. Only defined if this
   * is a bijection.
   */
  public Map inverse()
  {
    Map inverse = new Map();
    Enumeration domain_elements = domainElements();
    while ( domain_elements.hasMoreElements() ) {
      Object key = domain_elements.nextElement();
      inverse.put( get(key), key );
    }
    return inverse;
  }


  /**
   * Returns the size of the domain.
   */
  public int size()
  {
    return this.table.size();
  }


  /**
   * True if the domain contains o.
   */
  public boolean contains( Object o )
  {
    return ( get(o) != null );
  }


  /**
   * True if the codomain contains o.
   */
  public boolean codomainContains( Object o )
  {
    Enumeration e = this.table.elements();
    while ( e.hasMoreElements() )
      if ( e.nextElement() == o )
        return true;
    return false;
  }


  /**
   * Returns a one-level copy. The hashtable is distinct,
   * but keys and values are not cloned. Only use this
   * where it doesn't matter whether they should be or not,
   * since I may make it clone them later.
   */
  public Object clone()
  {
    Map copy = new Map();
    copy.unite( this );
    return copy;
  }


  /**
   * Displays the mapping, showing each domain element and
   * its image.
   */
  public String toDisplay()
  {
    StringBuffer sb = new StringBuffer();
    sb.append( "{" );
    Enumeration keys = this.table.keys();
    while ( keys.hasMoreElements() ) {
      Object key = keys.nextElement();
      sb.append( " " );
      sb.append( key.toString() );
      sb.append( "->" );
      sb.append( this.table.get(key).toString() );
      if ( keys.hasMoreElements() )
        sb.append( "," );
      if ( this.table.size() > 0 )
        sb.append( " " );
    }
    sb.append( "}" );
    return sb.toString();
  }

  /** 
   * As toDisplay, but adds the class name.
   */
  public String toString()
  {
    return "[Map " + toDisplay() + "]";
  }
}
