/* Sort.java */


package kb7.structures;

import java.util.Vector;
import kb7.general.Bug;


/**
 * This class defines a method for sorting a Vector. The
 * elements of the Vector can either be Strings, Integers or
 * Doubles, in which case these types' compareTo method is used to compare them, 
 * or can implement interface Comparable and define their own
 * compareTo method.
 * <P>
 * Some care is needed here. Java 1.2 apparently has its own
 * Comparable interface. So if you forget to import kb7.structures.Comparable,
 * you may end up referring to the Java 1.2 one (if running under
 * Java 1.2), and getting class cast errors.
 * <P>
 * There is also a version of sort which takes a comparison
 * predicate, implemented as a Comparer, as an extra argument. This
 * makes it easier to compare things that don't implement
 * Comparable. 
 */
public class Sort
{
  public static void main( String[] args )
  {
    Vector v = new Vector();
    v.addElement( "doggerel" );
    v.addElement( "dog" );
    v.addElement( "God" );
    v.addElement( "melon" );
    v.addElement( "melange" );
    v.addElement( "gourd" );
    v.addElement( "gourd" );
    v.addElement( "ace" );
    v.addElement( "ice" );
    System.out.println( v );
    sort( v );
    System.out.println( v );

    v = new Vector();
    v.addElement( new Integer(-1) );
    v.addElement( new Integer(1) );
    v.addElement( new Integer(-20) );
    v.addElement( new Integer(20) );
    v.addElement( new Integer(0) );
    System.out.println( v );
    sort( v );
    System.out.println( v );
  }


  public static final void sort( Vector v )
  {
    /* This code was posted to comp.lang.java.programmer by
       Gary Knopp (gknopp@ascend.com) on March 31st 1999.
       It's a quicksort.
    */
    qsort( v, 0, v.size()-1 );
  }


  private static final void qsort(Vector v,int first,int last)
  {
    int low=first;
    int high=last;

    if ( first>=last )
      return;

    Object mid = v.elementAt( (first+last)/2 );

    do {
      while ( compareTo( v.elementAt(low), mid ) < 0 )
        low++;
      while ( compareTo( v.elementAt(high), mid ) > 0 )
        high--;
      if ( low<=high ) {
        Object temp = v.elementAt(low);
        v.setElementAt( v.elementAt(high), low++ );
        v.setElementAt( temp, high-- );
      }
    } while (low<=high);
    qsort( v, first, high );
    qsort( v, low, last );
  }


  public static final void sort( Vector v, Comparer c )
  {
    /* This code was posted to comp.lang.java.programmer by
       Gary Knopp (gknopp@ascend.com) on March 31st 1999.
       It's a quicksort.
    */
    qsort( v, 0, v.size()-1, c );
  }


  private static final void qsort( Vector v, int first, int last, Comparer comparer )
  {
    int low=first;
    int high=last;

    if ( first>=last )
      return;

    Object mid = v.elementAt( (first+last)/2 );

    do {
      while ( comparer.compare( v.elementAt(low), mid ) < 0 )
        low++;
      while ( comparer.compare( v.elementAt(high), mid ) > 0 )
        high--;
      if ( low<=high ) {
        Object temp = v.elementAt(low);
        v.setElementAt( v.elementAt(high), low++ );
        v.setElementAt( temp, high-- );
      }
    } while (low<=high);
    qsort( v, first, high, comparer );
    qsort( v, low, last, comparer );
  }


  private static final int compareTo( Object a, Object b )
  {
    if ( a instanceof String && b instanceof String )
      return ((String)a).compareTo( (String)b );
    else if ( a instanceof Integer && b instanceof Integer )
      return ((Integer)a).compareTo( (Integer)b );
    else if ( a instanceof Double && b instanceof Double )
      return ((Double)a).compareTo( (Double)b );
    else if ( a instanceof kb7.structures.Comparable && b instanceof kb7.structures.Comparable )
      return ((kb7.structures.Comparable)a).compareTo( (kb7.structures.Comparable)b );
    else
      throw new Bug( "Illegal arguments: " + a + " " + b );
  }
}
