/* IntegerComboField.java */


package kb7.swing;


/**
 * This class behaves like an IntegerField. However, if right-clicked,
 * it displays a popup menu. If the user selects one of the menu's
 * options, then it is looked up in a translation table associated
 * with the options. If the value was set as a simple numeric string
 * such as "1234", the corresponding value is displayed in
 * the field and stored as its 'text' property. If the value is an
 * integer preceded by an operator such as "+" or "*", then it
 * is applied to the field's current value, and the result is stored
 * as the 'text' property. For example, a value added by doing
 *   add( "Triple it", "*3" )
 * will, when selected, multiply the field's contents by 3.
 *
 * The class adds the following methods to IntegerField:
 *   setOptions( MenuOptions options )
 * kb7.swing.MenuOptions represents an ordered sequence of menu options
 * which can also be used as a translation table. The idea is that
 * if these are used in implementing a menu, then the menu's caller
 * should be able to obtain either the text selected by the user, or
 * its translation. (In addition, the WOM compiler generates MenuOptions
 * from menu-option specifiers in WOM source code.)
 * 
 * The constructors for TextComboField allow either initial text, or
 * options, or both, to be supplied.
 *
 * IMPLEMENTATION
 *
 * This delegates all the work to the class ComboFieldDelegate, which
 * binds the field and its menu together, propagating menu-popup-events
 * to the menu, and menu selections back to the field. 
 */ 
public class IntegerComboField extends IntegerField  
{
  private ComboFieldDelegate delegate;


  public static void main( String argv[] )
  {
    IntegerComboField icf = 
      new IntegerComboField( "0", 
                             new MenuOptions().add( "One", "1" ).
                                               add( "Two", "2" ).
                                               add( "Three", "3" ).
                                               add( "Double it", "*2" ).  
                                               add( "Triple it", "*3" ).
                                               add( "Add 1", "+1" ).
                                               add( "Add 2", "+2" )
                           );
    icf.setColumns(5);
    
    javax.swing.JPanel p = new javax.swing.JPanel();
    p.add( icf );

    javax.swing.JFrame f = new javax.swing.JFrame();
    f.addWindowListener( new kb7.swing.BasicWindowMonitor() );
    f.getContentPane().add( p );
    f.setSize( 200, 200 );
    f.pack();
    f.setVisible( true );
  }


  public IntegerComboField()
  {
    super();
    this.delegate = new ComboFieldDelegate( this, ComboFieldOperation.INTEGER );
  }

  public IntegerComboField( MenuOptions options )
  {
    this();
    setOptions( options );
  }

  public IntegerComboField( String text )
  { 
    this();
    setTextValue( text ); 
  }

  public IntegerComboField( String text, MenuOptions options )
  {
    this( text );
    setOptions( options );
  }


  public final void setOptions( MenuOptions options )
  {
    this.delegate.setOptions( options );
  }
}

