/* MenuOptions.java */


package kb7.swing;

import java.util.Enumeration;
import java.util.Vector;


/**
 * This class is used to hold the options of a Wom menu.
 * It provides the same signature as Hashtable (so that
 * you can use it as a look-up table), but stores its
 * elements in the sequence they were added, which for
 * menu options is often wanted.
 * 
 * It has some extra "add" methods which are like the "put"
 * methods, but add an element and then return the MenuOptions
 * object itself. This is so the Wom compiler can generate
 * initialisation expressions like
 *   new MenuOptions().add(a,b).add(c,d)
 * by chaining calls of "add" together, given an
 *   {/ a:b, c:d /} construct.
 * 
 * It has some extra "put" and "putRange" methods for
 * putting options with integer and real values.
 * 
 * It has a "times" method for rescaling all values.
 * This is used, for example, where we want to rescale a
 * whole set of values, e.g. by converting from weekly
 * incomes to monthly incomes, but without altering the keys.
 * This returns a MenuOptions for the same reason the
 * "add" methods do.
 * 
 * Note: we ought to provide a Map interface for kb7.structures,
 * and make this implement that.
 */
public class MenuOptions
{
  private Vector contents = new Vector();


  /** 
   * Returns the number of options.
   */
  public int size()
  {
    return this.contents.size();
  }


  /**
   * Returns the ith option.
   */
  public MenuOption getOption( int i )
  {
    return (MenuOption)this.contents.elementAt(i);
  }


  /** 
   * All the 'put' methods add a key-value pair.
   */
  public void put( Object key, Object value )
  {
    this.contents.addElement( new MenuOption(key,value) );
  }

  public void put( Object key, int value )
  {
    put( key, new Integer(value) );
  }

  public void put( Object key, long value )
  {
    put( key, new Long(value) );
  }

  public void put( Object key, float value )
  {
    put( key, new Float(value) );
  }

  public void put( Object key, double value )
  {
    put( key, new Double(value) );
  }

  public void put( Object key, boolean value )
  {
    put( key, new Boolean(value) );
  }

  public void put( Object key, char value )
  {
    put( key, new Character(value) );
  }

  public void put( int key )
  {
    put( String.valueOf(key), new Integer(key) );
  }

  public void put( long key )
  {
    put( String.valueOf(key), new Long(key) );
  }

  public void put( float key )
  {
    put( String.valueOf(key), new Float(key) );
  }

  public void put( double key )
  {
    put( String.valueOf(key), new Double(key) );
  }

  public void put( boolean key )
  {
    put( String.valueOf(key), new Boolean(key) );
  }

  public void put( char key )
  {
    String s = String.valueOf(key);
    put( s, s  );
  }

  public void put( Object key )
  {
    put( key, key );
  }


  /** 
   * Adds a range of integers, as defined by the obvious loop.
   */
  public void putRange( int From, int To, int Step )
  {
    for ( int i=From; i<=To; i=i+Step )
      put( i );
  }

  /** 
   * Adds a range of integers with step size 1.
   */
  public void putRange( int From, int To )
  {
    putRange( From, To, 1 );
  }

  /** 
   * Adds a range of floats. All elements in the range will become
   * floats.
   */
  public void putRange( float From, float To, float Step )
  {
    for ( float f=From; f<=To; f=f+Step )
      put( f );
  }

  /** 
   * Adds a range of doubles. All elements in the range will become
   * doubles.
   */
  public void putRange( double From, double To, double Step )
  {
    for ( double d=From; d<=To; d=d+Step )
      put( d );
  }


  /**
   * Replaces key's old value with 'value', without changing
   * the order of elements. Use with care, in case the options
   * are shared.
   */
  public void replace( Object key, Object value )
  {
    for ( int i=0; i<size(); i++ ) {
      MenuOption pair = (MenuOption)this.contents.elementAt(i);
      if ( key.equals(pair.getKey()) ) {
        pair.setValue( value );
      }
    }
  }


  /** 
   * Returns the value corresponding to a key.
   * Null if not found.
   */
  public Object get( Object key )
  {
    for ( int i=0; i<size(); i++ ) {
      MenuOption pair = (MenuOption)this.contents.elementAt(i);
      if ( key.equals(pair.getKey()) )
        return pair.getValue();
    }
    return null;
  }


  /** 
   * Deletes a key/value pair.
   */
  public Object remove( Object key )
  {
    for ( int i=0; i<size(); i++ ) {
      MenuOption pair = (MenuOption)this.contents.elementAt(i);
      if ( key.equals(pair.getKey()) ) {
        this.contents.removeElement( pair );
        return key;
      }
    }
    return null;
  }


  /** 
   * True if there are no options.
   */
  public boolean isEmpty()
  {
    return size()==0;
  }


  /** 
   * Returns an enumeration of all the keys.
   */
  public Enumeration domainElements()
  {
    Vector keys = new Vector();
    for ( int i=0; i<size(); i++ ) {
      MenuOption pair = (MenuOption)this.contents.elementAt(i);
      keys.addElement( pair.getKey() );
    }
    return keys.elements();
  }


  /** 
   * Returns an enumeration of all the values.
   */
  public Enumeration codomainElements()
  {
    Vector elements = new Vector();
    for ( int i=0; i<size(); i++ ) {
      MenuOption pair = (MenuOption)this.contents.elementAt(i);
      elements.addElement( pair.getValue() );
    }
    return elements.elements();
  }


  /** 
   * Returns the vector of options, as a vector of key/value
   * pairs represented by a MenuOption.
   * Modify it with care!
   */
  public Vector options()
  {
    return this.contents;
  }


  /** 
   * Adds a key/value pair.
   */
  public MenuOptions add( Object key, Object value )
  {
    put( key, value );
    return this;
  }

  public MenuOptions add( Object key, int value )
  {
    put( key, value );
    return this;
  }

  public MenuOptions add( Object key, long value )
  {
    put( key, value );
    return this;
  }

  public MenuOptions add( Object key, float value )
  {
    put( key, value );
    return this;
  }

  public MenuOptions add( Object key, double value )
  {
    put( key, value );
    return this;
  }

  public MenuOptions add( Object key, boolean value )
  {
    put( key, value );
    return this;
  }

  public MenuOptions add( Object key, char value )
  {
    put( key, value );
    return this;
  }

  public MenuOptions add( Object key )
  {
    put( key );
    return this;
  }

  public MenuOptions add( int key )
  {
    put( key );
    return this;
  }

  public MenuOptions add( long key )
  {
    put( key );
    return this;
  }

  public MenuOptions add( float key )
  {
    put( key );
    return this;
  }

  public MenuOptions add( double key )
  {
    put( key );
    return this;
  }

  public MenuOptions add( boolean key )
  {
    put( key );
    return this;
  }

  public MenuOptions add( char key )
  {
    put( key );
    return this;
  }


  /** 
   * Adds a range of integers, by the obvious loop.
   */
  public MenuOptions addRange( int From, int To, int Step )
  {
    putRange( From, To, Step );
    return this;
  }

  /** 
   * Adds a range of integers with step size 1.
   */
  public MenuOptions addRange( int From, int To )
  {
    putRange( From, To );
    return this;
  }

  /** 
   * Adds a range of floats. All elements in the range will become
   * floats.
   */
  public MenuOptions addRange( float From, float To, float Step )
  {
    putRange( From, To, Step );
    return this;
  }

  /** 
   * Adds a range of doubles. All elements in the range will become
   * doubles.
   */
  public MenuOptions addRange( double From, double To, double Step )
  {
    putRange( From, To, Step );
    return this;
  }


  /** 
   * Multiplies all values by i, but does not alter the keys.
   * Makes integer and long values long, and float and double
   * values double.
   */
  public MenuOptions times( long l )
  {
    for ( int i=0; i < this.contents.size(); i++ ) {
      ((MenuOption)this.contents.elementAt(i)).times(l);
    }
    return this;
  }

  /** 
   * Same as above: makes integer and long values long, and float
   * and double values double.
   */
  public MenuOptions times( int l )
  {
    for ( int i=0; i < this.contents.size(); i++ ) {
      ((MenuOption)this.contents.elementAt(i)).times(l);
    }
    return this;
  }

  /** 
   * Same intention as above: makes all values double.
   */
  public MenuOptions times( double d )
  {
    for ( int i=0; i < this.contents.size(); i++ ) {
      ((MenuOption)this.contents.elementAt(i)).times(d);
    }
    return this;
  }

  /** 
   * Same as above: makes all values double.
   */
  public MenuOptions times( float f )
  {
    for ( int i=0; i < this.contents.size(); i++ ) {
      ((MenuOption)this.contents.elementAt(i)).times(f);
    }
    return this;
  }


  public String toString()
  {
    StringBuffer s = new StringBuffer();
    s.append( "[MenuOptions " );
    for ( int i=0; i < this.contents.size(); i++ ) {
      MenuOption mo = (MenuOption)this.contents.elementAt(i);
      s.append( mo.toShortString() );
      s.append( " " );
    }
    s.append( "]" );
    return s.toString();
  }
}
