/* RealField.java */


package kb7.swing;

import javax.swing.text.Document; 
import kb7.general.Bug;


/**
 * This field only allows digits and an optional
 * decimal point to be typed into it, allowing
 * it to be used for (non-exponential-notation)
 * real numbers. The syntax of permitted strings 
 * is
 *   digit+ | digit* "." digit+ | digit+ "." digit*
 * The field refuses to update if non-digits are
 * typed into it, or a decimal point is when it
 * already contains one. When ENTER is pressed, if
 * the field is empty, or there is only a decimal
 * point in it, it will report an error. If the
 * string starts or ends with a decimal point, 
 * pressing ENTER will cause a "0" to be prepended
 * or appended respectively.
 *   
 * Methods added to TextField are
 *   getdoubleValue() and setdoubleValue(d),
 * which return and set the text as a double. 
 * 
 * Also, setValue and getValue are overridden to
 * be compatible with these. setValue takes an Object,
 * casts it to Double, and calls setdoubleValue on the
 * double value. getValue returns a Double wrapping
 * the result of getdoubleValue.
 * 
 * IMPLEMENTATION
 *
 * The field delegates its checking to RealFieldFormat
 * via the hooks in CheckingField.
 */
public class RealField extends CheckingField 
{
  public static void main( String[] argv )
  {
    RealField rf1 = new RealField();
    rf1.setdoubleValue(1.2345);
    rf1.setColumns(5);

    RealField rf2 = new RealField();
    rf2.setTextValue("2.3456");
    rf2.setColumns(5);

    javax.swing.JPanel panel = new javax.swing.JPanel();
    panel.add( rf1 );
    panel.add( rf2 );

    javax.swing.JFrame frame = new javax.swing.JFrame();
    frame.addWindowListener( new BasicWindowMonitor() );
    frame.setContentPane( panel );
    frame.setSize( 350, 400 );
    frame.setVisible( true );
  }


  public RealField() 
  { super( new RealFieldFormat() ); }

  public RealField( double value )
  {
    this();
    setdoubleValue( value );
  }

  public RealField( String text )
  {
    this();
    setTextValue( text );
  }  


  public final double getdoubleValue() 
  {
    double d = 0;
    try {
      d = new Double( getTextValue() ).doubleValue();
    }
    catch ( Exception e ) {
      throw new Bug( "Unexpected number format exception: " + e );
    }
    return d;
  }

  public final void setdoubleValue( double value ) 
  {
    setTextValue( value+"" );
  }


  /**
   * If the value is a Double, use its doublevalue. Otherwise,
   * convert to string, and hope the result makes sense as a
   * double.
   */
  public void setValue( Object value )
  {
    if ( value instanceof Double )
      setdoubleValue( ((Double)value).doubleValue() );
    else 
      setdoubleValue( new Double( value.toString() ).doubleValue() );
  }

  public Object getValue()
  {
    return new Double( getdoubleValue() );
  }
}
