/* CheckingField.java */


package kb7.swing;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import javax.swing.text.Document;
import javax.swing.JOptionPane;
import javax.swing.JTextField; 
import kb7.structures.Pair;


/**
 * This is the base class for fields which check their
 * text when the field loses focus. The field creates
 * an ActionListener. When focus is lost or ENTER is pressed, this
 * listener gets the field's text and calls checkText on
 * it. This can return either an error message or null.
 * If it returns an error message, the listener reports
 * it in a JOptionPane. If the error message is null,
 * the listener then calls standardiseText. This can
 * standardise the text by, for example, truncating
 * fixed-point numbers that are too long, or converting
 * months in dates to standard form. The listener then
 * inserts this text back into the field.
 *
 * The field adds the methods below. These will be of 
 * interest to implementors; users won't be using this
 * class except via its subclasses.
 *
 *  protected void reportError( String error, boolean hard )
 *    Called if checkText returns a non-null string.
 *    Reports the error in a JOptionPane modal dialog
 *    box. Subclasses may want to override this. If
 *    hard, the error is reported as an error, otherwise
 *    as a warning.
 *
 *  public void setTextValue( String text )
 *    Overrides the superclass's setTextValue. This new
 *    version calls checkText and standardiseText. 
 *    This is a check against the programmer accidentally
 *    initialising the field by calling setTextValue with
 *    an invalid string.
 *
 *  protected void setTextNoChecks( String text )
 *    This calls the superclass's old setTextValue method,
 *    without the calls to check and standardise the
 *    text. Use this, e.g., from inside a checking
 *    method, where a call to setTextValue might lead to
 *    an infinite loop. 
 *
 * This class also adds setLenient and getLenient. These
 * implement a boolean property indicating whether errors
 * should be reported as errors or warnings.
 */
public abstract class CheckingField extends TextField 
{
  private boolean is_lenient = true;
  // If false, reject text that fails the checks; if true,
  // warn but accept it.

  protected FieldConstraint constraints = null;
  // If not null, contains constraints which are applied
  // once it is known that the text is syntactically valid.

  protected FieldFormat format = null;
  // Contains a Document which checks characters as they
  // are typed; a checkText method which applies global
  // checks once the user presses ENTER; and a 
  // standardiseText method which may reformat it once
  // checked. To set the format, pass it to the protected
  // constructor below.


  protected CheckingField( FieldFormat f ) 
  {  
    super( f.document );
    this.format = f;

    addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) 
      { 
        CheckingField input = (CheckingField)e.getSource();
        String text = input.getTextValue();
        setTextValue( text );
      }
    } );   

    addFocusListener(new FocusAdapter() {
      public void focusLost(FocusEvent e) 
      { 
        if ( !e.isTemporary() ) {
          CheckingField input = (CheckingField)e.getSource();
          String text = input.getTextValue();
          setTextValue( text );
        }
      }
    } );
  }


  public void setTextValue( String text )
  {
    if ( this.format == null ) {
      super.setTextValue( text );
      return;
    }
    // The format should never be null. This
    // conditional is just in case setText gets called
    // from one of the superclass constructors, 
    // before the format has been initialised.

    String error = this.format.checkText( text );

    if ( error != null ) {
      reportError( error, true );
      return; 
    }

    error = null;
    Pair constraint_results = null;
    if ( this.constraints != null ) { 
      constraint_results = this.constraints.check( text, this.is_lenient );
      if ( constraint_results != null )
        error = (String)constraint_results.getFirst();
    }

    if ( error != null && this.is_lenient ) {
      reportError( error, false );
      text = this.format.standardiseText( text );
      setTextNoChecks( text );
      fireValueEvent( new ValueAcceptedEvent( this ) );
    }
    else if ( error != null && !this.is_lenient ) {
      reportError( error, true );
      text = (String)constraint_results.getSecond();
      setTextNoChecks( text ); 
      fireValueEvent( new ValueAcceptedEvent( this ) );
    }
    else {
      text = this.format.standardiseText( text );
      setTextNoChecks( text );
      fireValueEvent( new ValueAcceptedEvent( this ) );
    }
  }


  public final void setLenient( boolean is_lenient )
  {
    this.is_lenient = is_lenient;
  }

  public final boolean getLenient()
  {
    return this.is_lenient;
  }


  protected void setTextNoChecks( String text )
  { 
    super.setTextValue( text );
  }


  protected void reportError( String error, boolean hard )
  {
    if ( hard )
      JOptionPane.showMessageDialog( this, error, "Error Message", JOptionPane.ERROR_MESSAGE );
    else
      JOptionPane.showMessageDialog( this, error, "Warning Message", JOptionPane.WARNING_MESSAGE );
//    grabFocus();
  } 
}
