/* ComboFieldDelegate.java */


package kb7.swing;

import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Enumeration;
import javax.swing.JPopupMenu;
import javax.swing.JMenuItem;
import kb7.general.Bug;


/**
 * This is an internal class used in implementing ComboFields. It
 * glues together a field and its popup menu. On the one hand, it
 * adds a mouse listener to the field, so that when right-clicked,
 * the menu appears. On the other, it adds a property-change
 * listener to the menu, so that when the user selects an item,
 * thus changing the menu's 'text' property, the change is fired
 * to the delegate, and hence can be used to update the field.
 *
 * Create a combo-field delegate when gluing a field to its popup
 * menu. Pass the delegate's constructor the field, and one of the
 * variables ComboFieldOperation.INTEGER or ComboFieldOperation.REAL, 
 * specifying the type of the numbers used (Integer or Double). 
 * The constructor will store the field, and create a new popup menu
 * which is also stores. It will also add a mouse listener from the
 * field to the menu, and a property-change listener from the menu to
 * itself.
 *
 * The combo field should define a setOptions method which calls the
 * delegate's setOptions. This method takes a MenuOptions. It runs
 * through them, replacing value strings starting with an operation
 * recognised by ComboFieldOperation with an instance of ComboFieldOperation
 * which, when applied to the correct type of argument, returns the
 * result of operating on it. In effect, this maps a menu option
 * such as ("Double it", "*2") to a mapping from the string "Double it"
 * to the function lambda x.x*2. If the user selects this option,
 * the function will be applied to the number represented by the
 * field's 'text' property. 
 *
 * IMPLEMENTATION
 *
 * The inner class PopupListener activates the popup menu whenever 
 * the user right-clicks on the field.
 *
 * The inner class MenuTextChangeListener updates the field's 'text'
 * property whenever the user selects a menu option. If this
 * option represents a value, it just replaces the old text. If it
 * represents an operation on the previous value, it applies this
 * operation to the value. In this case, the corresponding MenuOption's
 * value will be an instance of ComboFieldOperation, representing
 * a function. Calling its applyTo method on the field's value
 * returns the result of applying this function to the value.
 *
 * As already mentioned, setOptions runs through its MenuOptions
 * argument, replacing value strings that start with an operator 
 * - "*2" or "+1.3" - by instances of ComboFieldOperation that 
 * implement the corresponding function.
 */
public class ComboFieldDelegate
{
  private TextField tf;
  // The text field to which the popup menu is attached.

  private TextPopupMenu popup;
  // The menu.

  private int type;
  // The field's type: integer or double.


  public ComboFieldDelegate( TextField tf, int type )
  {
    this.tf = tf;
    this.popup = new TextPopupMenu();
    this.popup.addValueListener( new MenuTextChangeListener() ); 
    this.tf.addMouseListener( new PopupListener() );
    this.type = type;
  }


  public final void setOptions( MenuOptions options )
  {
    Enumeration elements = options.domainElements();
    while ( elements.hasMoreElements() ) {
      Object key = elements.nextElement(); 

      Object value = options.get( key );
      // The value can be any type, but if it's ComboFieldOperation we treat
      // it specially, and when converting to text, we assume toString()
      // is defined.

      if ( value instanceof ComboFieldOperation )
        throw new Bug( "Option value for key " + key + 
                       " is already a ComboFieldOperation: " + value + ". " +
                       "This may mean that the same key occurs twice." 
                     );
        // Because if a value for this key has already been processed,
        // the next time the key is looked up in this loop, we'll find
        // not a string but a ComboFieldOperation.                      

      if ( ComboFieldOperation.hasOperator(value.toString(),this.type) ) {
        /* Assume the value represents a function to apply to the field's value. 
           Replace it by an implementation of this function.
        */
        ComboFieldOperation action = 
          ComboFieldOperation.createComboFieldOperation( value.toString(), this.type ); 
        options.replace( key, action );
      }
        /* Otherwise leave the value alone. */
    }

    this.popup = new TextPopupMenu( options );
    this.popup.addValueListener( new MenuTextChangeListener() ); 
    // I don't know how to remove all the options from a popup menu,
    // so the assignment above just creates a new menu and gives it the
    // new options.
  }


  private final class MenuTextChangeListener implements ValueListener
  {
    public void valueAccepted( ValueAcceptedEvent ev )
    {
      TextField tf = ComboFieldDelegate.this.tf;
      if ( ev.getSource() == ComboFieldDelegate.this.popup ) {
        Object value = ComboFieldDelegate.this.popup.getValue();
        if ( value instanceof ComboFieldOperation ) {
          ComboFieldOperation action = (ComboFieldOperation)value;
          Object new_value = action.applyTo( tf.getValue() );
System.out.println( "Calling updateValueFromAction" );
          tf.updateValueFromAction( action.op, new_value );
        }
        else
          tf.setTextValue( value.toString() );
      }
    }
  }


  private final class PopupListener extends MouseAdapter   
  {
    public PopupListener()
    { }


    public void mousePressed( MouseEvent e ) 
    {
      maybeShowPopup(e);
    }

    public void mouseReleased( MouseEvent e ) 
    {
      maybeShowPopup(e);
    }

    private void maybeShowPopup( MouseEvent e ) 
    {
      if ( e.isPopupTrigger() ) {
        ComboFieldDelegate.this.popup.show( e.getComponent(),
                                            e.getX(), e.getY()
                                          );
      }
    }
  }
}

