/* ComboFieldOperation.java */


package kb7.swing;

import java.util.Hashtable;
import kb7.general.Bug;


/**
 * When setting menu key-value pairs for combo fields, the caller can
 * specify values that start with an operator, e.g. "*2". These represent
 * a function (here lambda x.x*2) to be applied to the field's existing
 * value if the user selects the key. Instances of class ComboFieldOperation
 * implement such functions. They can be stored as the value part of a 
 * MenuOperation, and define a method applyTo which, applied to the
 * field's old value, return the result of applying this function.
 * 
 * The public methods are:
 *   static ComboFieldOperation createComboFieldOperation( String fn, int type )
 *     The string should be something representing a function to be applied
 *     to a combo-field's value, e.g "+2" or "*3.5". The integer should be
 *     ComboFieldOperation.INTEGER or ComboFieldOperation.REAL. This method 
 *     returns a representation of the function: in effect, "+2" is converted
 *     into lambda x.x+2, for example.
 *     Operators currently recognised are "+" and "*", for both integer and
 *     real.
 *  
 *   Object applyTo( Object arg )
 *     Applies the function represented by the instance to the argument. The
 *     argument and result are Object, so that they can be used ploymorphically.
 *
 *   static boolean hasOperator( String fn, int type )
 *     The arguments should be as for createComboFieldOperation( fn, type ).
 *     This method returns true if the string starts with an operator 
 *     recognised by createComboFieldOperation for that type.
 */
public abstract class ComboFieldOperation
{
  public static int INTEGER = 0;
  public static int REAL = 1;
  // The public type constants.

  protected String op;
  // The operator name.


  public static void main( String[] argv )
  {
    ComboFieldOperation action = ComboFieldOperation.createComboFieldOperation( "*27", INTEGER );
    System.out.println( action );
    System.out.println( action.applyTo( new Integer(3) ) );
    System.out.println( ComboFieldOperation.hasOperator("*2",INTEGER) ); 
    System.out.println( ComboFieldOperation.hasOperator("2",INTEGER) );

    action = ComboFieldOperation.createComboFieldOperation( "*0.27", REAL );
    System.out.println( action );
    System.out.println( action.applyTo( new Double(300.1) ) );
    System.out.println( ComboFieldOperation.hasOperator("*0.21",INTEGER) ); 
    System.out.println( ComboFieldOperation.hasOperator("0.21",INTEGER) );
  }


  public ComboFieldOperation()
  { }


  public static final ComboFieldOperation createComboFieldOperation( String s, int type )
  {
    if ( type == INTEGER ) 
      return IntegerComboFieldOperation.createIntegerComboFieldOperation( s, type );
    else if ( type == REAL ) 
      return RealComboFieldOperation.createRealComboFieldOperation( s, type );
    else
      throw new Bug( "Illegal type: " + type );
  }


  public static boolean hasOperator( String s, int type )
  {
    if ( type == INTEGER ) 
      return IntegerComboFieldOperation.hasOperator( s, type );
    else if ( type == REAL ) 
      return RealComboFieldOperation.hasOperator( s, type );
    else
      throw new Bug( "Illegal type: " + type );
  }


  public abstract Object applyTo( Object arg2 );


  public static final Object applyTo( ComboFieldOperation action, Object arg2 )
  {
    return action.applyTo(arg2);
  }  


  /* This inner class represents a function on integers. It has one subclass
     for each kind of function, e.g. add and multiply.
  */
  private abstract static class IntegerComboFieldOperation extends ComboFieldOperation
  {
    private static Hashtable action_classes = new Hashtable();
    static { action_classes.put( "+", addInteger.class ); 
             action_classes.put( "-", subtractInteger.class ); 
             action_classes.put( "*", multiplyInteger.class );
           }
    // Maps an operator symbol to the corresponding subclass of 
    // IntegerComboFieldOperation.

    protected int arg;
    // The argument following the operator symbol in the original string.
    // E.g. 2 from "*2".    

    /* Called by the superclass's createComboFieldOperation method.
    */
    public static final IntegerComboFieldOperation createIntegerComboFieldOperation( String s, int type )
    {
      /* Get the operator. */
      String op = s.substring( 0, 1 );
      
      /* What's the subclass of IntegerComboFieldOperation that this represents? 
         If none, the operator is illegal.
      */
      Class operator_class = (Class)action_classes.get( op );
      if ( operator_class == null )
        throw new Bug( "Illegal operator in menu action string: " + s );

      /* Get the rest of the string. Parse as an integer, and throw
         an error if it's illegal. Store as 'arg' in an instance of the appropriate
         subclass of IntegerComboFieldOperation.
      */
      String arg = s.substring(1);
      try {
        IntegerComboFieldOperation closure = (IntegerComboFieldOperation)operator_class.newInstance();
        closure.op = op;
        closure.arg = Integer.parseInt(arg);
        return closure;
      }
      catch ( NumberFormatException e ) {
        throw new Bug( "Illegal integer in menu action string: " + s );
      }  
      catch ( IllegalAccessException e ) {
        throw new Bug( "Error creating instance: " + e );
      } 
      catch ( InstantiationException e ) {
        throw new Bug( "Error creating instance: " + e );
      } 
    }

    public static final boolean hasOperator( String s, int type )
    {
      String op = s.substring( 0, 1 );
      Class operator_class = (Class)action_classes.get( op );
      return operator_class != null;
    }
  }


  /* This inner class represents a function on integers. It has one subclass
     for each kind of function, e.g. add and multiply.
  */
  private abstract static class RealComboFieldOperation extends ComboFieldOperation
  {
    private static Hashtable action_classes = new Hashtable();
    static { action_classes.put( "+", addReal.class ); 
             action_classes.put( "-", subtractReal.class ); 
             action_classes.put( "*", multiplyReal.class );
           }
    // Maps an operator symbol to the corresponding subclass of 
    // RealComboFieldOperation.

    protected double arg;
    // The argument following the operator symbol in the original string.
    // E.g. 2.3 from "*2.3".    

    public static final RealComboFieldOperation createRealComboFieldOperation( String s, int type )
    {
      /* Get the operator. */
      String op = s.substring( 0, 1 );

      /* What's the subclass of IntegerComboFieldOperation that this represents? 
         If none, the operator is illegal.
      */
      Class operator_class = (Class)action_classes.get( op );
      if ( operator_class == null )
        throw new Bug( "Illegal operator in menu action string: " + s );

      /* Get the rest of the string. Parse as a real, and throw
         an error if it's illegal. Store as 'arg' in an instance of the appropriate
         subclass of RealComboFieldOperation.
      */
      String arg = s.substring(1);
      try {
        RealComboFieldOperation closure = (RealComboFieldOperation)operator_class.newInstance();
        closure.op = op;
        closure.arg = new Double(arg).doubleValue();
        return closure;
      }
      catch ( NumberFormatException e ) {
        throw new Bug( "Illegal real in menu action string: " + s );
      }  
      catch ( IllegalAccessException e ) {
        throw new Bug( "Error creating instance: " + e );
      } 
      catch ( InstantiationException e ) {
        throw new Bug( "Error creating instance: " + e );
      } 
    }

    public static final boolean hasOperator( String s, int type )
    {
      String op = s.substring( 0, 1 );
      Class operator_class = (Class)action_classes.get( op );
      return operator_class != null;
    }
  }


  /* This class represents the function lambda x.x+arg, where arg is
     its 'arg' instance variable, and the arguments and result are
     integers.
  */
  private static class addInteger extends IntegerComboFieldOperation
  {
    public addInteger()
    { }

    public addInteger( int arg )
    { this.arg = arg; }

    public Object applyTo( Object arg2 )
    {
      return new Integer( ((Integer)arg2).intValue() + this.arg );
    }
  }


  /* This class represents the function lambda x.x-arg, where arg is
     its 'arg' instance variable, and the arguments and result are
     integers.
  */
  private static class subtractInteger extends IntegerComboFieldOperation
  {
    public subtractInteger()
    { }

    public subtractInteger( int arg )
    { this.arg = arg; }

    public Object applyTo( Object arg2 )
    {
      return new Integer( ((Integer)arg2).intValue() - this.arg );
    }
  }


  /* This class represents the function lambda x.x*arg, where arg is
     its 'arg' instance variable, and the arguments and result are
     integers.
  */
  private static class multiplyInteger extends IntegerComboFieldOperation
  {
    public multiplyInteger()
    { }

    public multiplyInteger( int arg )
    { this.arg = arg; }

    public Object applyTo( Object arg2 )
    {
      return new Integer( ((Integer)arg2).intValue() * this.arg );
    }
  }  


  /* This class represents the function lambda x.x+arg, where arg is
     its 'arg' instance variable, and the arguments and result are
     doubles.
  */
  private static class addReal extends RealComboFieldOperation
  {
    public addReal()
    { }

    public addReal( double arg )
    { this.arg = arg; }

    public Object applyTo( Object arg2 )
    {
      return new Double( ((Double)arg2).doubleValue() + this.arg );
    }
  }


  /* This class represents the function lambda x.x-arg, where arg is
     its 'arg' instance variable, and the arguments and result are
     doubles.
  */
  private static class subtractReal extends RealComboFieldOperation
  {
    public subtractReal()
    { }

    public subtractReal( double arg )
    { this.arg = arg; }

    public Object applyTo( Object arg2 )
    {
      return new Double( ((Double)arg2).doubleValue() - this.arg );
    }
  }


  /* This class represents the function lambda x.x*arg, where arg is
     its 'arg' instance variable, and the arguments and result are
     doubles.
  */
  private static class multiplyReal extends RealComboFieldOperation
  {
    public multiplyReal()
    { }

    public multiplyReal( double arg )
    { this.arg = arg; }

    public Object applyTo( Object arg2 )
    {
      return new Double( ((Double)arg2).doubleValue() * this.arg );
    }
  }
}



