/* FixedPointField.java */


package kb7.swing;

import javax.swing.text.Document; 
import kb7.general.Bug;


/**
 * This field only allows digits and an optional
 * decimal point to be typed into it, allowing
 * it to be used for fixed-point real numbers. 
 * The syntax of permitted strings is
 *   digit+ | digit* "." digit+ | digit+ "." digit*
 * where no more than After digits are allowed 
 * after the decimal point. The default for After
 * is 2.
 *
 * The field refuses to update if non-digits are
 * typed into it, or a decimal point is when it
 * already contains one, or more than After digits
 * are typed after the point. When ENTER is pressed, if
 * the field is empty, or there is only a decimal
 * point in it, it will report an error. If the
 * string starts or ends with a decimal point, 
 * pressing ENTER will cause a "0" to be prepended
 * or appended respectively. In addition, as many
 * "0"'s will be appended to the end as are necessary
 * to make up After digits. If the number has too
 * many digits after the point, the extra ones
 * will be deleted. Note that this rounds the
 * number down, not towards the nearest integer.
 *
 * Methods added to TextField are
 *   getdoubleValue() and setdoubleValue(d),
 * which return and set the text as a double. 
 *
 * Also, setValue and getValue are overridden to
 * be compatible with these. setValue takes an Object,
 * casts it to Double, and calls setdoubleValue on the
 * double value. getValue returns a Double wrapping
 * the result of getdoubleValue.
 * 
 * IMPLEMENTATION
 *
 * The field delegates its checking to FixedPointFieldFormat
 * via the hooks in CheckingField. 
 */
public class FixedPointField extends CheckingField 
{
  public static void main( String[] argv )
  {
    FixedPointField fpf1 = new FixedPointField();
    fpf1.setAfter(2);
    fpf1.setdoubleValue(1.2);
    fpf1.setColumns(5);

    FixedPointField fpf2 = new FixedPointField();
    fpf2.setAfter(4);
    fpf2.setTextValue("2.345");
    fpf2.setColumns(5);

    javax.swing.JPanel panel = new javax.swing.JPanel();
    panel.add( fpf1 );
    panel.add( fpf2);

    javax.swing.JFrame frame = new javax.swing.JFrame();
    frame.addWindowListener( new BasicWindowMonitor() );
    frame.setContentPane( panel );
    frame.setSize( 350, 400 );
    frame.setVisible( true );
  }


  public FixedPointField() 
  { super( new FixedPointFieldFormat() ); }

  public FixedPointField( double value )
  {
    this();
    setdoubleValue( value );
  }

  public FixedPointField( String text )
  {
    this();
    setTextValue( text );
  }  


  public final void setAfter( int after )
  { ((FixedPointFieldFormat)this.format).setAfter(after); }

  public final int getAfter()
  { return ((FixedPointFieldFormat)this.format).getAfter(); }


  public final void setdoubleValue( double value ) 
  {
    setTextValue( value + "" );
  }

  public final double getdoubleValue() 
  {
    double d = 0;
    try {
      d = new Double( getTextValue() ).doubleValue();
    }
    catch ( Exception e ) {
      throw new Bug( "Unexpected number format exception: " + e );
    }
    return d;
  }


  /**
   * If the value is a Double, use its doublevalue. Otherwise,
   * convert to string, and hope the result makes sense as a
   * double.
   */
  public void setValue( Object value )
  {
    if ( value instanceof Double )
      setdoubleValue( ((Double)value).doubleValue() );
    else 
      setdoubleValue( new Double( value.toString() ).doubleValue() );
  }

  public Object getValue()
  {
    return new Double( getdoubleValue() );
  }
}
