/* IntegerField.java */


package kb7.swing;

import javax.swing.text.Document; 
import kb7.general.Bug;


/**
 * This field only allows digits to be typed into it,
 * allowing it to be used for validating integers.
 * The syntax of permitted strings is
 *   digit+ 
 * The field refuses to update if non-digits are
 * typed into it. When ENTER is pressed, if
 * the field is empty, it will report an error. 
 *
 * Methods added to TextField are
 *   getintValue() and setintValue(i),
 * which return and set the text as an integer. 
 * 
 * Also, setValue and getValue are overridden to
 * be compatible with these. setValue takes an Object,
 * casts it to Integer, and calls setintValue on the
 * integer value. getValue returns an Integer wrapping
 * the result of getintValue.
 *
 * IMPLEMENTATION
 *
 * The field delegates its checking to IntegerFieldFormat
 * via the hooks in CheckingField.
 */
public class IntegerField extends CheckingField 
{
  public static void main( String[] argv )
  {
    IntegerField i1 = new IntegerField();
    i1.setintValue(1);
    i1.setColumns(5);

    IntegerField i2 = new IntegerField();
    i2.setTextValue("2");
    i2.setColumns(5);

    i1.addValueListener( new ValueListener() {
                           public void valueAccepted( ValueAcceptedEvent ev ) 
                           {
                             System.out.println( ev );
                           }
    } );
    i2.addValueListener( new ValueListener() {
                           public void valueAccepted( ValueAcceptedEvent ev ) 
                           {
                             System.out.println( ev );
                           }
    } );

    javax.swing.JPanel panel = new javax.swing.JPanel();
    panel.add( i1 );
    panel.add( i2 );

    javax.swing.JFrame frame = new javax.swing.JFrame();
    frame.addWindowListener( new BasicWindowMonitor() );
    frame.setContentPane( panel );
    frame.setSize( 350, 400 );
    frame.setVisible( true );
  }


  public IntegerField() 
  { super( new IntegerFieldFormat() ); } 

  public IntegerField( int value )
  {
    this();
    setintValue( value );
  }

  public IntegerField( String text )
  {
    this();
    setTextValue( text );
  }  


  public void setintValue( int value ) 
  {
    setTextValue( value+"" );
  }

  public int getintValue() 
  {
    int retVal = 0;
    try {
      retVal = Integer.parseInt( getTextValue() );
    } 
    catch ( Exception e ) {
      throw new Bug( "Unexpected number format exception: " + e );
    }
    return retVal;
  }


  /**
   * If the value is an Integer, use its intvalue. Otherwise,
   * convert to string, and hope the result makes sense as an
   * integer.
   */
  public void setValue( Object value )
  {
    if ( value instanceof Integer )
      setintValue( ((Integer)value).intValue() );
    else 
      setintValue( Integer.parseInt( value.toString() ) );
  }

  public Object getValue()
  {
    return new Integer( getintValue() );
  }
}
