/* MenuOption.java */


package kb7.swing;

import kb7.general.Bug;


/**
 * This class represents one menu option, i.e.
 * a key-value pair. It has access methods for getting
 * the key and value. It is used in MenuOptions.
 * 
 * It also has a "times" method for multiplying the
 * value by a factor without altering the key. This is
 * used to implement "times" in MenuOptions, where
 * we want to rescale a whole set of values, e.g. by
 * converting from weekly incomes to monthly incomes, but
 * without altering the keys.
 *
 * The toString method just returns the key. This allows
 * a MenuOption to be used as an item in the list of a
 * JComboBox (the Java Swing component). The default
 * Swing renderer displays the option by calling its
 * toString method, so will just return the key. But the
 * option also contains the corresponding value, so 
 * can be used as part of a table that translates keys
 * to values.
 */
public final class MenuOption
{
  private Object key, value;


  /** 
   * Constructor.
   */
  public MenuOption( Object key, Object value )
  {
    this.key = key;
    this.value = value;
  }


  /** 
   * Returns the key.
   */
  public final Object getKey()
  {
    return this.key;
  }

  /** 
   * Sets the key.
   */
  public final void setKey( Object key )
  {
    this.key = key;
  }


  /** 
   * Returns the value.
   */
  public final Object getValue()
  {
    return this.value;
  } 

  /** 
   * Sets the value.
   */
  public final void setValue( Object value )
  {
    this.value = value;
  }


  /** 
   * Multiplies the value by i, but does not alter the key.
   * Makes integer and long values long, and float and double
   * values double.
   */
  public final void times( long l )
  {
    if ( this.value instanceof Integer || this.value instanceof Long )
      this.value = new Long( ((Number)this.value).longValue()*l );
    else if ( this.value instanceof Float || this.value instanceof Double )
      this.value = new Double( ((Number)this.value).doubleValue()*l );
    else
      throw new Bug( "value must be a number for times" );
  }

  /** 
   * Same as above: makes integer and long values long, and float
   * and double values double.
   */
  public final void times( int i )
  {
    times( (long)i );
  }

  /** 
   * Same intention as above: makes all values double.
   */
  public final void times( double d )
  {
    if ( this.value instanceof Number )
      this.value = new Double( ((Number)this.value).doubleValue()*d );
    else
      throw new Bug( "value must be a number for times" );
  }

  /** 
   * Same as above: makes all values double.
   */
  public final void times( float f )
  {
    times( (double)f );
  }


  public final String toString()
  {
    return this.key.toString();
  }


  public final String toShortString()
  {
    return this.key + "|->" + this.value;
  }
}
