/* TextComboBox.java */


package kb7.swing;


/** 
 * This combo box extends JComboBox with a few methods that
 * we find handy in WOM. We use it as the base class for
 * IntegerEditableComboBox etc.
 *
 * The added methods are:
 *   setTextValue(String), getTextValue()
 *   setValue(Object), getValue()
 *
 * setTextValue and getTextValue are analogous to setText and 
 * getText in JTextField. getTextValue returns the text last selected
 * by the user, either from the list of menu options or from
 * whatever was typed into the field. setTextValue changes the
 * selected text, modifying the appearance of the cmobo box
 * so that it appears as the current selected item.
 *
 * Every JComboBox has methods for adding items to the list
 * of menu options. We define a class MenuOption, which is a
 * key/value pair. These allow the combo box to be used as a
 * lookup table, so that the user can select (for example) a
 * word for a number, but the programmer can get back the 
 * corresponding number. For example, in the main method 
 * below, the four MenuOptions added will each appear when
 * the combo box is rendered just as the key - "One", "Two"
 * etc. This is because their toString method just returns the
 * key, and a combo box's renderer calls toString by default.
 * The important use of this is in kb7.wom.client.TextComboBox.
 *
 * Calling getValue will search the list of menu options.
 * It checks the selected text against each option's key, and
 * if it matches, returns the corresponding value. If none
 * match, it returns the text. It ignores options that aren't 
 * instances of MenuOption.
 *
 * setValue works in the other direction. If its parameter
 * is equal to the value of any menu option, it will set 
 * the text to the corresponding key. (If several options
 * have the same key, the effect is undefined.)
 *
 * Note: the above specification may need rethinking. I'm not
 * using these components at the moment.
 */
public class TextComboBox extends javax.swing.JComboBox
{
  private String text;


  public static void main( String[] argv )
  {
    final TextComboBox cb = new TextComboBox();
    cb.addItem( new MenuOption( "One", "1" ) );
    cb.addItem( new MenuOption( "Two", "2" ) );
    cb.addItem( new MenuOption( "Three", "3" ) );
    cb.addItem( "4" );
    cb.addItem( new MenuOption( "Five", "5" ) );

    cb.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed( java.awt.event.ActionEvent e ) {
        System.out.println( "User selected text " + cb.getTextValue() );
        System.out.println( "Value = " + cb.getValue() );
        System.out.println();
      }
    });

    javax.swing.JPanel panel = new javax.swing.JPanel();
    panel.setLayout( new java.awt.FlowLayout() );
    panel.add( cb );

    final javax.swing.JTextField key = new javax.swing.JTextField(10);
    panel.add( key );
    key.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed( java.awt.event.ActionEvent e ) {
        System.out.println( "Setting key to " + key.getText() );
        cb.setTextValue( key.getText() );
        System.out.println();
      }
    });

    final javax.swing.JTextField value = new javax.swing.JTextField(10);
    panel.add( value );
    value.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed( java.awt.event.ActionEvent e ) {
        System.out.println( "Setting value to " + value.getText() );
        cb.setValue( value.getText() );
        System.out.println();
      }
    });

    javax.swing.JFrame frame = new javax.swing.JFrame();
    frame.setSize( 300, 400 );
    frame.setContentPane( panel );
    frame.pack();
    frame.addWindowListener( new kb7.swing.BasicWindowMonitor() );
    frame.setVisible( true );  
  }
  

  public TextComboBox( javax.swing.JTextField field )
  {
    setEditor( new Editor(field) );  
  }

  public TextComboBox()
  { 
    super( new javax.swing.DefaultComboBoxModel() );
    setEditable(true);
    setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
    addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed( java.awt.event.ActionEvent e ) {
        TextComboBox cb = (TextComboBox)e.getSource();
        cb.text = cb.getSelectedItem().toString();
      }
    });
  }


  public final void setTextValue( String text )
  {
    ((javax.swing.JTextField)getEditor().getEditorComponent()).setText( text );
    // We make setting the text affect the text field. If 'text' is
    // in the list of options, this also seems to affect the box so that
    // when the user next clicks on the select-from-list button, the
    // corresponding item in the list is selected. This is good.
    this.text = text;
    setSelectedItem( text );
  }

  public final String getTextValue()
  {
    return this.text;
  }


  /**
   * Sets the text from value. If value is equal to the value of one of
   * the menu options, sets the text to the corresponding key, thus
   * performing a reverse translation. Otherwise, sets it to value.
   */
  public final void setValue( Object value )
  {
    for ( int i=0; i<getItemCount(); i++ ) {
      Object item = getItemAt(i);
      if ( item instanceof MenuOption && ((MenuOption)item).getValue().equals(value) ) {
        setTextValue( ((MenuOption)item).getKey().toString() );
        return;
      }
    }
    setTextValue( value.toString() );      
  }

  /** 
   * Returns the text after translating. If it is equal to the key of one of
   * the menu options, returns the corresponding value. Otherwise, returns
   * the text.
   */
  public final Object getValue()
  {
    String key = getTextValue();
    for ( int i=0; i<getItemCount(); i++ ) {
      Object item = getItemAt(i);
      if ( item instanceof MenuOption && ((MenuOption)item).getKey().equals(key) )
        return ((MenuOption)item).getValue();
    }
    return getTextValue();
  }


  public final String toString()
  {
    return "[TextComboBox]";
  }


  /* For setting other fields. Thanks to 'wiarda' on the Sun JDC Forum
     for showing how to do this.
  */
  private class Editor extends javax.swing.plaf.basic.BasicComboBoxEditor
  {
    public Editor( javax.swing.JTextField tf )
    {
      super();
      this.editor = tf;
    }
  }
}
