/* TextComboField.java */


package kb7.swing;

import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import javax.swing.JPopupMenu;
import javax.swing.JMenuItem;


/**
 * This class behaves like a TextField. However, if right-clicked,
 * it displays a popup menu. If the user selects one of the menu's
 * options, then it is looked up in a translation table associated
 * with the options, and the corresponding value is displayed in
 * the field and stored as its "text" property.
 *
 * The class adds the following methods to TextField:
 *   setOptions( MenuOptions options )
 * kb7.swing.MenuOptions represents an ordered sequence of menu options
 * which can also be used as a translation table. The idea is that
 * if these are used in implementing a menu, then the menu's caller
 * should be able to obtain either the text selected by the user, or
 * its translation. (In addition, the WOM compiler generates MenuOptions
 * from menu-option specifiers in WOM source code.)
 * 
 * The constructors for TextComboField allow either initial text, or
 * options, or both, to be supplied.
 *
 * IMPLEMENTATION
 *
 * The field contains a TextPopupMenu. This is a subclass of JPopupMenu,
 * which stores MenuOptions so they can be used as a translation
 * table. We listen for right-clicks, and pop up this menu when they 
 * appear. The menu has a bound property, "text", which when changed,
 * fires a listener (MenuTextChangeListener) declared here. That 
 * replaces the text in this field by the translated menu option.
 */ 
public class TextComboField extends TextField  
{
  private TextPopupMenu popup;


  public static void main( String argv[] )
  {
    TextComboField tcf = 
      new TextComboField( "0", new MenuOptions().add("One","1").add("Two","2").add("Three","3") );
    tcf.setColumns(5);
    
    javax.swing.JPanel p = new javax.swing.JPanel();
    p.add( tcf );

    javax.swing.JFrame f = new javax.swing.JFrame();
    f.addWindowListener( new kb7.swing.BasicWindowMonitor() );
    f.getContentPane().add( p );
    f.setSize( 200, 200 );
    f.pack();
    f.setVisible( true );
  }


  public TextComboField()
  {
    super();
    this.popup = new TextPopupMenu();
    this.popup.addValueListener( new MenuTextChangeListener() ); 
    addMouseListener( new PopupListener() );
  }

  public TextComboField( MenuOptions options )
  {
    this();
    setOptions( options );
  }

  public TextComboField( String text )
  { 
    this();
    setTextValue( text ); 
  }

  public TextComboField( String text, MenuOptions options )
  {
    this( text );
    setOptions( options );
  }


  public final void setOptions( MenuOptions options )
  {
    this.popup = new TextPopupMenu( options );
    this.popup.addValueListener( new MenuTextChangeListener() ); 
  }


  private final class MenuTextChangeListener implements ValueListener
  {
    public void valueAccepted( ValueAcceptedEvent ev )
    {
      if ( ev.getSource() == TextComboField.this.popup ) {
        setTextValue( (String)TextComboField.this.popup.getValue() );
      }
    }
  }


  private final class PopupListener extends MouseAdapter   
  {
    public PopupListener()
    { }


    public void mousePressed( MouseEvent e ) 
    {
      maybeShowPopup(e);
    }

    public void mouseReleased( MouseEvent e ) 
    {
      maybeShowPopup(e);
    }

    private void maybeShowPopup( MouseEvent e ) 
    {
      if ( e.isPopupTrigger() ) {
        TextComboField.this.popup.show( e.getComponent(),
                                        e.getX(), e.getY()
                                      );
      }
    }
  }
}
