/* TextPopupMenu.java */


package kb7.swing;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Enumeration;
import javax.swing.JPopupMenu;
import javax.swing.JMenuItem;
import javax.swing.event.EventListenerList;


/**
 * This class is used in implementing TextComboField, but
 * may also be useful for other purposes. It extends
 * JPopupMenu to provide a menu with an ordered sequence of
 * options which can also be used as a translation table.
 * Once the user selects an option, its text can be obtained
 * with getTextValue(); its text after lookup in the translation
 * table can be obtained with getValue().
 *
 * Methods added to JPopupMenu are:
 *   void setOptions( MenuOptions options )
 *     kb7.swing.MenuOptions represents an ordered sequence of menu 
 *     options which can also be used as a translation table. The 
 *     idea is that if these are used in implementing a menu, then 
 *     the menu's caller should be able to obtain either the text 
 *     selected by the user, or its translation. (In addition, 
 *     the WOM compiler generates MenuOptions from menu-option 
 *     specifiers in WOM source code.)
 *
 *   String getTextValue()
 *     Returns the last selected text. If none has yet been selected,
 *     returns null.
 *
 *   Object getValue()
 *     Returns the last selected text, after translating in the MenuOptions.
 *     This will usually be a string, but can be any object. If no 
 *     option has yet been selected, returns null.
 *
 *   void addValueListener( ValueListener )
 *   void removeValueListener( ValueListener )
 */   
public class TextPopupMenu extends JPopupMenu  
{
  private MenuOptions options;
  private String text = null;
  private EventListenerList value_listeners = new EventListenerList();


  public static void main( String argv[] )
  {
    javax.swing.JPanel p = new javax.swing.JPanel();
    TextPopupMenu m = new TextPopupMenu( new MenuOptions().add("One","1").add("Two","2").add("Three","3") );
    p.add( m );

    m.addValueListener( new ValueListener() {
                          public void valueAccepted( ValueAcceptedEvent ev ) 
                          {
                            System.out.println( ev );
                          }
    } );

    javax.swing.JFrame f = new javax.swing.JFrame();
    f.addWindowListener( new kb7.swing.BasicWindowMonitor() );
    f.getContentPane().add( p );
    f.setSize( 200, 200 );
    f.pack();
    f.setVisible( true );
  }


  public TextPopupMenu()
  {
    super();
  }

  public TextPopupMenu( MenuOptions options )
  {
    this();
    setOptions( options );
  }


  public final void setOptions( MenuOptions options )
  {
    this.options = options;
    Enumeration elements = options.domainElements();
    while ( elements.hasMoreElements() ) {
      Object key = elements.nextElement();
      JMenuItem item = new JMenuItem( key.toString() );
      add( item );
      item.addActionListener( new item_listener() );
    }
  }


  public final String getTextValue()
  {
    return this.text;
  }


  public final Object getValue()
  {
    if ( this.text == null )
      return null;
    else
      return this.options.get(this.text);
  }


  public final synchronized void addValueListener( ValueListener l )
  {
    this.value_listeners.add( ValueListener.class, l );
  }

  public final synchronized void removeValueListener( ValueListener l )
  {
    this.value_listeners.remove( ValueListener.class, l );
  }

  protected final void fireValueEvent( ValueAcceptedEvent ev )
  {
    Object[] listeners = this.value_listeners.getListenerList();
    for ( int i=0; i<this.value_listeners.getListenerCount(); i+=2 ) {
      if ( listeners[i] == ValueListener.class ) 
        ( (ValueListener)listeners[i+1] ).valueAccepted( ev );
    }
  }


  /* An instance of this will be called when the user selects an option.
     It updates the text, and fires a property change event for it.
  */
  private final class item_listener implements ActionListener
  {
    public final void actionPerformed( ActionEvent e )
    {
      JMenuItem source = (JMenuItem)e.getSource();
      String old_text = TextPopupMenu.this.text;
      String new_text = source.getText();
      TextPopupMenu.this.text = new_text;
      TextPopupMenu.this.fireValueEvent( new ValueAcceptedEvent( TextPopupMenu.this ) );        
    }
  }
}
