(module-name 'ModuleDependencies)

(module-static #t)


; This program analyses the dependencies between
; Kawa modules for 'require'd classes, and generates
; a script to compile the modules in the correct
; order. Thus it makes up for the lack of a Kawa
; equivalent to the 'javac *.java' command.
;
; To use it, call 
;   (generate-script CLASSNAME SCRIPT-fILE)
; e.g.
;   (generate-script 'kb7.mm3.general.CellNames "compile.bat")
; You will need to change the code of generate-script
; to cater for your own preferences on the root of 
; your package directory and the form of your compilation
; command.


(require <kb7.kawa.general.pregexp>)


; Generates a script to compile all files required
; for classname, including classname's own file. All
; classes are subpackages of kb7.* , with the \kb7\ directory 
; directly under d: . Puts the script in to-file.
;
; Example:
;   (generate-script 'kb7.mm3.web.NextPage "fred.bat")
;
(define (generate-script (classname <symbol>)
                         (to-file <string>)) <void>
  (let* ( (script "")

          (root "d:\\")
          ; We assume all classes are under the same outer-level
          ; package, kb7 . This is the directory _above_ the
          ; directory which holds the outer-level package, i.e.
          ; above \kb7\. You will want to change this root.

          (classnames (preorder classname root))
          ; The important part. Names of required classes, 
          ; most basic first. We iterate through these, generating
          ; a command for each one.

          (eol (invoke-static <java.lang.System> 'getProperty 'line.separator  (string->symbol "\n")))
       )

    ; Convert the classnames to a script.
    (for-each (lambda (classname)
                (let* ( (filename <string> (classname->filename classname root))
                        ; Name of the file holding this class.

                        (split <list> (split-classname classname))

                        (prefix <string> (car split))
                        ; Prefix to use in the -P option to Kawa compile command.    
                      )

                  (set! script (string-append script
                                              "java kawa.repl -d \\. -P " prefix " -C " filename eol 
                               )
                  )
                  ; You may want to change this command.

                )
              )
              classnames
    )

    ; Save the script.
    (with-output-to-file to-file 
                         (lambda () 
                           (format "rem (generate-script \"%S\" \"%S\")" classname to-file)
                           (display script) (newline)
                         )
    )
  )
)


; Returns a list of all classes required by classname, in
; preorder (i.e. a preorder traversal of the dependency graph,
; such that any class occurs before all those on which
; it depends). Assumes no cycles. 'root' is the name of the directory
; above the outer level of the package, written with a trailing
; separator after its name.
;
; Example:
;   (preorder 'kb7.mm3.web.NextPage "d:\\")
;
(define (preorder (classname <symbol>) 
                  (root <string>)) <list>

  ; This algorithm is taken from the topological sort in section 6.6
  ; of "Data Structures and Algorithms" by Aho, Hopcroft and Ullman
  ; (Addison-Wesley 1983).

  (let ( (visited? <java.util.Hashtable> (make <java.util.Hashtable>))
         (stack <list> '())
         (result <list> '())
         ; We build up the final, topologically sorted, list
         ; here.
       )

    (define (dfs (classname <symbol>)) <void>
      (let ( (children <list> (read-and-extract-required-classes (classname->filename classname root)))
           )
        (set! stack (cons classname stack))
        (invoke visited? 'put classname #t)
        (for-each (lambda ((child <symbol>))
                    (when (eq? (invoke visited? 'get child) #!null)
                      (dfs child)
                      (set! result (append result (list (car stack))))
                      (set! stack (cdr stack))                    
                    )
                  )
                  children
        )           
      )
    )

    (dfs classname)

    (append result (list classname))

  )

)


; Converts a classname to the full filename of the Kawa source
; file that defines the class. 
;
; Example:
;   (classname->filename 'kb7.mm3.web.NextPage "d:\\")
;
(define (classname->filename (classname <symbol>) 
                             (root <string>)) <string>
  (string-append root   
                 (dots-to-slashes (symbol->string classname))                 
                 ".scm"
  )
)


; Replaces dots by "/" or "\" (the path separator) in name.
;
; Example:
;   (dots-to-slashes "kb7.mm3.web.NextPage")
;
(define (dots-to-slashes (name <string>)) <string>
  (let ( (path-separator <string> (string-append (static-field <java.io.File> 'separator)))
       )
    (pregexp-replace* "[.]" 
                      name 
                      (if (equal? path-separator "\\")
                        "\\\\"
                      ;else
                        path-separator
                      )
    )
    ; Using pregexp-replace for string replacement has the
    ; disadvantage that \ is an escape character, so we need
    ; to unescape it, even in the replacement string.  
  )
)


; Splits a classname PREFIX1 "." PREFIX2 "." PREFIX-n "." NAME
; into the PREFIX1 ... PREFIX-n "." and the NAME parts,
; returning the result as a list
;   (PREFIX1 ... PREFIX-n "." part, NAME part)
;
; Example:
;   (split-classname 'a.b.c.d)
; returns
;   ("a.b.c." "d") .
;   
(define (split-classname (classname <symbol>)) <list>
  (let ( (match <list> (pregexp-match (string-append "((?:[[:alnum:]]+[.])+)"  ; Repeated (PREFIX ".") parts.
                                                     "([[:alnum:]]+)"          ; The final NAME.
                                                                               ; Both bracketed so the matched portions
                                                                               ; appear in the match results.
                                      )
                                      (symbol->string classname)
                       )
         )
       )
    (list (cadr match) (caddr match))
  )
)


; Reads file with full filename f, assuming it
; to be a Kawa source file, and searches it for expressions 
; of the form
;   (require CLASSNAME)
; and returns a list of the classnames.
;
; Example:
;   (read-and-extract-required-classes "d:\\kb7\\mm3\\web\NextPage.scm")
;
(define (read-and-extract-required-classes (filename <string>)) <list>
  (display "Reading ") (display filename) (newline)
  (try-catch 
    (extract-required-classes (readall (open-input-file filename)))
  ; exceptionally
    (ex <java.lang.Throwable>
      (begin
        (display "Error reading file") (newline)
        '()
      )
    )
  )
)


; Searches s-exprs for expressions of the form
;   (require CLASSNAME)
; and returns a list of the classnames. These 
; are held as symbols (so that hashtable lookups are
; guaranteed to work), but without the enclosing
; pointed brackets.
;
; Example:
;   (extract-required-classes '( (module-name test) (require <kb7.mm3.general.Bugs>) ) )
;
(define (extract-required-classes (s-exprs <list>)) <list>
  (if (eq? s-exprs '())
    '()
  ;else
    (let* ( (s-expr (car s-exprs))
            (require? <boolean> (if (list? s-expr)
                                  (and (eq? (length s-expr) 2)
                                       (eq? (car s-expr) 'require)
                                  )
                                ;else
                                  #f
                                )
            )
          )
      (if require?
        (let ( (classname-as-string <string> (symbol->string (cadr s-expr)))
             )
          (cons (string->symbol (substring classname-as-string 1 (- (string-length classname-as-string) 1)))
                (extract-required-classes (cdr s-exprs))
          )
        )
      ;else
        (extract-required-classes (cdr s-exprs))
      )
    )
  )
)


; Returns a list of the S-expressions
; left on port p.
;
(define (readall (p <input-port>)) <list>
  (let ( (item (read p))
       )
    (if (eq? item #!eof)
      '()
    ;else
      (cons item (readall p))
    )
  )
)
