/* FastIncludes.java */


package kb7.wsm;

import java.io.*;


/** 
 * This class handles simple server-side
 * includes. The main method is doIncludes,
 * which we call from Control.scm . 
 */
public class FastIncludes
{
  /**
   * This method expands server-side includes
   * in text, assumed to be the output from
   * a .pfn page. The includes can only be of 
   * the form 
   *  <!--#include virtual="/ssi/virtual/top.ssi"-->
   * with no extra layout and where the argument
   * is a local URL. This URL will be converted
   * to a filename whose contents will be
   * read and recursively expanded before
   * being substituted for the original 
   * directive.
   */
  public final static void doIncludes( String text, String document_root, PrintWriter out )
                           throws java.io.IOException
  {
    doIncludes( new StringBuffer(text), document_root, out );
  }


  public final static void doIncludes( StringBuffer text, String document_root, PrintWriter out )
                           throws java.io.IOException
  {
    int i = 0;
    while ( true ) {
      int open_pos = find( text, "<!--#include virtual=\"", i );

      if ( open_pos == -1 ) {
        out.print( text.substring(i) );
        return;
      }
      
      out.print( text.substring(i,open_pos) );

      // Index of first character of <!--#include virtual=" .
      // This substring is 22 characters long.

      int close_pos = find( text, "\"-->", open_pos+22 );
      if ( close_pos == -1 )
        throw new RuntimeException( "No closing --> for server-side include." );
      // Index of first character of "--> . 
      // This substring is 4 characters long.

      String uri = text.substring( open_pos+22, close_pos );
      uriToContents( uri, document_root, out );

      i = close_pos+4;
    }
  }


  /* Returns the index of the first occurrence of sub in text
     starting from start, or -1.
  */
  private final static int find( StringBuffer text, String sub, int start )
  {
    int i = start;
    int text_len = text.length();
    int sub_len = sub.length();
    while ( i + sub_len <= text_len ) {
      if ( startsWith( text, sub, i ) ) 
        return i;
      i++;
    }
    return -1;
  }


  /* True if text has substring sub starting at start.
     We assume sub would not extend beyond the end of text.
  */
  private final static boolean startsWith( StringBuffer text, String sub, int start )
  {
    int first_non = start+sub.length();
    for ( int i=start; i<first_non; i++ )
      if ( text.charAt(i) != sub.charAt(i-start) )
        return false;
    return true;
  }


  /* Reads the contents of the local file denoted by uri.
  */
  private final static void uriToContents( String uri, String document_root, PrintWriter out )
                            throws java.io.IOException
  {
    String filename = Utilities.joinDirToFile( document_root, uri );
    FileInputStream in = new FileInputStream( filename );

    StringBuffer contents = new StringBuffer();
    int ch;
    while ( (ch = in.read()) != -1 )
      contents.append( (char)ch );
    in.close();

    doIncludes( contents, document_root, out );
  }
}
