/* Utilities.java */


package kb7.wsm;

import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import gnu.lists.LList;
import gnu.lists.Pair;


/**
 * This class defines assorted utilities for handling
 * files, filenames, URLs, exceptions, and strings.
 */
public final class Utilities
{
  /**
   * The end-of-line character.
   */
  public final static String eol = System.getProperty( "line.separator", "\n" );


  /**
   * If filename does not contain an extension, appends extension.
   * If it does not contain a directory, prepends dir.
   * The tests for these are a bit crude.
   */
  public final static String makeFullFilename( String filename, String dir, String extension )
  {
    if ( !filename.endsWith(extension) )
      filename += extension;
    if ( filename.indexOf( File.separator ) == -1 )
      filename = dir + filename;
    return filename; 
  }


  /**
   * Writes a string to a file. Does not add a newline at the end.
   */
  public final static void stringToFile( String filename, String s )
                           throws IOException
  {
    PrintWriter out = new PrintWriter( new FileWriter(filename) );
    out.print( s );
    out.close();
  }


  /**
   * Reads a file and returns the entire contents as a string.
   */
  public final static String fileToString( String filename )
                             throws IOException
  {
    int c;
    StringBuffer sb = new StringBuffer();
    FileReader in = new FileReader(filename);
    while ( ( c = in.read() ) != -1 ) {
      sb.append( (char)c );
    }
    in.close();
    return sb.toString();
  }


  /**
   * Returns the result of replacing \ in f by the
   * file separator character for this system. Only defined
   * because there seem to be problems with calling
   * 'replace' with char argumentss directly from Kawa.
   */
  public final static String replaceFileSeparator( String f )
  {
    if ( File.separatorChar == '/' ) 
      return f;
    else 
      return f.replace( '/', File.separatorChar );
  }


  /**
   * Appends file to dir, ensuring that there is only one
   * file separator character between them. dir may or
   * may not end with one; file may or may not start with
   * one.
   */
  public final static String joinDirToFile( String dir, String file )
  {
    if ( !dir.endsWith( File.separator ) )
     dir += File.separatorChar;
    if ( file.startsWith(File.separator) )
     file = file.substring( 1 );
    return dir + file;      
  }


  /**
   * Implements the WSM function get-inputs. Given a map from HTTP parameter
   * names to their values, and a prefix, returns a new map from all parameter
   * names that have this prefix (but drops the prefix) to their values. 
   * Only defined rather than in Control.scm because Kawa crashes if we try to 
   * invoke the Enumeration methods from it.
   */
  public final static Hashtable getInputs( Hashtable input_map, String prefix )
  {
    Hashtable values = new Hashtable();
    Enumeration parameter_names = input_map.keys();
    while ( parameter_names.hasMoreElements() ) {
      String parameter_name = (String)parameter_names.nextElement();
      if ( parameter_name.startsWith(prefix) ) 
        values.put( drop(parameter_name,prefix), input_map.get(parameter_name) );
    }
    return values;
  }


  /** 
   * Returns a hashtable mapping parameter names to values.
   * The values are stored as a list, if there's more than one, otherwise
   * as the value itself. This is by far the most common case, so
   * worth treating specially.    
   * Only defined rather than in Control.scm because Kawa crashes if we try to 
   * invoke the Enumeration methods from it.
   */
  public final static Hashtable getParameterValues( HttpServletRequest req )
  {
    Hashtable result = new Hashtable();
    Enumeration names = req.getParameterNames();
    while ( names.hasMoreElements() ) {
      String name = (String)names.nextElement();
      String[] values = req.getParameterValues(name);
      result.put( name, convertParameterValues(values) );
    }
    return result;
  }


  /**
   * Converts values, the parameter values for a parameter
   * name, to the way we store them in our inputs map.
   * If values contains one value, returns it, otherwise
   * returns a list of them.
   */
  private final static Object convertParameterValues( String[] values )
  {
    if ( values.length == 1 )
      return values[0];
    else {
      LList result = LList.Empty;
      for ( int i=0; i<values.length; i++ )
        result = new Pair( values[i], result );
      return result;
    }
  }        
// NOTE!!! Should have customisable test here for
// servers that put the session id as a query parameter,
// e.g. JServe puts a query parameter called JServSessionIdjp
// (the "jp" may be peculiar to the user). If so,
// should remove it so that the isEmpty test works.


  /**
   * Encodes a URL using res. Only defined here
   * because Kawa can't invoke that method directly,
   * either.
   */
  public final static String encodeUrl( HttpServletResponse res, String url )
  {
    return res.encodeUrl( url );
  }


  /**
   * Removes the portion of s upto and including the
   * first delimiter. Undefined if delimiter doesn't
   * occur in s.
   */  
  public final static String drop( String s, String delimiter ) 
  { 
    int delimiter_pos = s.indexOf(delimiter);
    return s.substring( delimiter_pos+delimiter.length() );
  }
}

