/* WSMServlet.java */


package kb7.wsm;

import java.io.*;
import java.util.Hashtable;
import java.util.Enumeration;
import javax.servlet.*;
import javax.servlet.http.*;
import gnu.lists.LList;
import gnu.lists.Pair;
import kawa.standard.Scheme;
import kb7.wsm.Control;


/**
 * This is the servlet for WSM. Most of the processing
 * is done by the Kawa code in Control.scm, called from
 * doGet. 
 * The servlet should be configured with an init parameter
 * "document-root", giving the top-level directory for
 * the .pfn files, and
 * "sitedefs-name", giving the name of the sitedefs file.
 * If omitted, the document root will be that taken from
 * the real path for the URI "/", and the sitedefs file
 * will be sitedefs.scm under the document root.
 */
public class WSMServlet extends HttpServlet
{
  public final void doPost( HttpServletRequest req, HttpServletResponse res )
                    throws ServletException, IOException
  {
    if ( req.getContentType().toLowerCase().startsWith("multipart/form-data") ) 
      go( req, res, getFilesAndOtherParameters(req) );
      // Since fileupload requires POST, we only override doPost.
    else
      go( req, res, Utilities.getParameterValues(req) );
  }


  public final void doGet( HttpServletRequest req, HttpServletResponse res )
                    throws ServletException, IOException
  {
    go( req, res, Utilities.getParameterValues(req) );
  }


  private final void go( HttpServletRequest req, HttpServletResponse res, Hashtable parameters )
  {
    Scheme.registerEnvironment();

    String document_root = getInitParameter("document-root");
    String sitedefs_name = getInitParameter("sitedefs-name");
    String local_sitedefs_name = getInitParameter("local-sitedefs-name");

    Control.handleGet( req, res, document_root, sitedefs_name, local_sitedefs_name, parameters );
  }


  /* This is the file upload code from Sun's servlet. 
     It "will handle file uploads, as performed by Netscape 3 and 4. 
     Note:  This program does not implement RFC 1867, merely a subset of it".
  */
  private final Hashtable getFilesAndOtherParameters( HttpServletRequest req )
                          throws ServletException, IOException
  {
        Hashtable table;

        if (req.getContentLength() > 1000000 ) 
          throw new RuntimeException( "Content too long - sent files are limited in size" );

        int ind = req.getContentType().indexOf("boundary=");
        if (ind == -1) 
            throw new RuntimeException( "boundary is not set" );

        String boundary = req.getContentType().substring(ind+9);

        if (boundary == null) 
            throw new RuntimeException( "boundary is not set" );

        table = parseMulti(boundary, req.getInputStream());

/*
// Diagnostics...
	for ( Enumeration fields = table.keys(); fields.hasMoreElements(); ) {
	    String name = (String)fields.nextElement();
	    System.err.println( name+":" );
	    Object obj = table.get(name);
	    if ( obj instanceof Hashtable ) {
		// It's a file!
		Hashtable filehash = (Hashtable)obj;
		System.err.println( "Filename: "+filehash.get("filename") );
		System.err.println( "Content-Type: "+filehash.get("content-type") );
                String contents = (String)filehash.get( "content" );
		System.err.println( "Contents: ");
		System.err.println( contents );
		System.err.println();
		System.err.println( "" );
	    } 
            else if ( obj instanceof String[] ) {
		String[] values = (String[]) obj;
		for (int i =0; i < values.length; i++) 
		    System.err.println( values[i] );
	    }
	}

	System.err.flush();
// ... end diagnostics.
*/

        return table;
  }  


  /* The meat of the file upload.  This method parses the input, and
     returns a hashtable of either LLists or String values (for parameters)
     or Hashtable values (for files uploaded).  The values of the entries
     in the hashtable are name, filename, Content-Type, and Contents.
     Note that uploads should be capped in size by the calling method, since 
     otherwise a denial of service attack on server memory becomes trivial.
  */
  private final Hashtable parseMulti( String boundary, ServletInputStream in ) 
                          throws IOException 
  {
       int buffSize = 1024*8; // 8K
       Hashtable hash = new Hashtable();
       int result;
       String line;
       String lowerline;
       String boundaryStr = "--"+boundary;
       ByteArrayOutputStream content;
       String filename;
       String contentType;
       String name;
       String value;

       byte[] b = new byte[buffSize];

       result = in.readLine(b,0,b.length);
       // failure.
       if (result == -1) 
           throw new IllegalArgumentException( "InputStream truncated" );
       line = new String(b,0,0,result);
       // failure.
       if ( !line.startsWith(boundaryStr) ) 
          throw new IllegalArgumentException( "MIME boundary missing: "+line );
       while (true) {
	   // Some initialization
	   filename = null;
	   contentType = null;
	   content = new ByteArrayOutputStream();
	   name = null;

           // get next line (should be content disposition)
           result = in.readLine(b,0,b.length);
           if (result == -1) return hash;
           line = new String(b,0,0,result-2);
	   lowerline = line.toLowerCase();
           if (!lowerline.startsWith("content-disposition"))
               // don't know what to do, so we'll keep looking...
               continue;
           // determine what the disposition is
           int ind = lowerline.indexOf("content-disposition: ");
           int ind2 = lowerline.indexOf(";");
	   if (ind == -1 || ind2 == -1) 
               throw new IllegalArgumentException( "Content Disposition line misformatted: "+line );
           String disposition = lowerline.substring(ind+21,ind2);
           if (!disposition.equals("form-data"))
               throw new IllegalArgumentException( "Content Disposition of "+disposition+" is not supported" );
           // determine what the name is
	   int ind3 = lowerline.indexOf("name=\"",ind2);
	   int ind4 = lowerline.indexOf("\"",ind3+7);
	   if (ind3 == -1 || ind4 == -1) 
               throw new IllegalArgumentException( "Content Disposition line misformatted: "+line );
           name = line.substring(ind3+6,ind4);
	   // determine filename, if any
	   int ind5 = lowerline.indexOf("filename=\"",ind4+2);
	   int ind6 = lowerline.indexOf("\"",ind5+10);
           if (ind5 != -1 && ind6 != -1) 
               filename = line.substring(ind5+10,ind6);

	   // Whew!  We now move onto the next line, which
	   // will either be blank, or Content-Type, followed by blank.
           result = in.readLine(b,0,b.length);
           if ( result == -1 ) 
             return hash;
           line = new String(b,0,0,result-2); // -2 to remove \r\n
	   lowerline = line.toLowerCase();
	   if (lowerline.startsWith("content-type")) {
	       int ind7 = lowerline.indexOf(" ");
	       if (ind7 == -1) 
                   throw new IllegalArgumentException( "Content-Type line misformatted: "+line );
	       contentType = lowerline.substring(ind7+1);
	       //  read blank header line
               result = in.readLine(b,0,b.length);
               if ( result == -1 ) 
                 return hash;
               line = new String(b,0,0,result-2); // -2 to remove \r\n
	       if (line.length() != 0) 
                   throw new IllegalArgumentException( "Unexpected line in MIMEpart header: "+line );
	   } 
           else if ( line.length() != 0 ) 
               throw new IllegalArgumentException( "Misformatted line following disposition: "+line );
            
           // Read content, implement readahead by one line.
	   boolean readingContent = true;
	   boolean firstLine = true;
	   byte[] buffbytes = new byte[buffSize];
	   int buffnum = 0;

           result = in.readLine(b,0,b.length);
           if ( result == -1 ) 
             return hash;
           line = new String(b,0,0,result); 
	   if ( !line.startsWith(boundaryStr) ) {
	       System.arraycopy(b,0,buffbytes,0,result);
	       buffnum = result;
               result = in.readLine(b,0,b.length);
               if ( result == -1 ) 
                 return hash;
               line = new String(b,0,0,result); 
	       firstLine = false;
	       if ( line.startsWith(boundaryStr) ) 
		   readingContent = false;
           } 
           else 
	       readingContent = false;

           while ( readingContent ) {
	       content.write(buffbytes,0,buffnum);
	       System.arraycopy(b,0,buffbytes,0,result);
	       buffnum = result;
               result = in.readLine(b,0,b.length);
               if ( result == -1 ) 
                 return hash;
               line = new String(b,0,0,result); 
	       if ( line.startsWith(boundaryStr) ) 
                 readingContent = false;
           }

	   if (!firstLine) {
	       // -2 to trim \r\n
	       if (buffnum>2)
	           content.write(buffbytes,0,buffnum-2); 
           }

	   // Now set appropriate variable, populate hashtable.
           // This, I think, is a normal query parameter and
           // not a file. We map the parameter name to either
           // a list (if more than one value) or a string.
           // It should be kept in step with the representation
           // returned by Utilities.convertParameterValues .
	   if ( filename == null ) {
	      if ( hash.get(name) == null ) 
	          hash.put( name,content.toString() );
               else {
		  Object prevobj = hash.get(name);
		  if ( prevobj instanceof Pair ) {
                      // Hashtable entry is for more than one value, and therefore
                      // already a list.
		      Pair prev = (Pair)prevobj;
		      hash.put( name, new Pair( content.toString(), prev ) );
		  } 
                  else if ( prevobj instanceof String ) {
                      // Hashtable entry is for one value, therefore is a string.
                      String prev = (String)prevobj;
                      hash.put( name, new Pair( content.toString(), new Pair( prev, LList.Empty ) ) );
                  }
                  else
		    // Now what? I think this breaks the standard.
		    throw new IllegalArgumentException(" failure in parseMulti hashtable building code" );
	      }
           } 
           else {
              // The parameter is a file.
	      // Yes, we don't return Hashtable[] for multiple files of same name.  AFAIK, that's not allowed.
	      Hashtable filehash = new Hashtable(4);
	      filehash.put("name",name);
	      filehash.put("filename",filename);
	      if (contentType == null) 
                contentType = "application/octet-stream";
	      filehash.put("content-type",contentType);
	      filehash.put("content",content.toString());
	      hash.put(name,filehash);
	   }
       }
   }


  public String getServletInfo()
  {
    return "WSM servlet";
  }
}
